@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -68,21 +68,25 @@ from pathlib import Path
 import click
 import click_completion
 import yaml
+from click_plugins import with_plugins
+from pkg_resources import iter_entry_points
 
 from renku.cli.clone import clone
 from renku.cli.config import config
 from renku.cli.dataset import dataset
 from renku.cli.doctor import doctor
 from renku.cli.exception_handler import IssueFromTraceback
-from renku.cli.githooks import githooks
-from renku.cli.init import init
+from renku.cli.githooks import githooks as githooks_command
+from renku.cli.graph import graph
+from renku.cli.init import init as init_command
 from renku.cli.log import log
-from renku.cli.migrate import migrate
+from renku.cli.migrate import check_immutable_template_files, migrate, migrationscheck
 from renku.cli.move import move
 from renku.cli.remove import remove
 from renku.cli.rerun import rerun
 from renku.cli.run import run
 from renku.cli.save import save
+from renku.cli.service import service
 from renku.cli.show import show
 from renku.cli.status import status
 from renku.cli.storage import storage
@@ -99,7 +103,7 @@ from renku.core.management.repository import default_path
 #: Monkeypatch Click application.
 click_completion.init()
 
-WARNING_UNPROTECTED_COMMANDS = ["init", "clone", "help"]
+WARNING_UNPROTECTED_COMMANDS = ["init", "clone", "service", "help"]
 
 
 def _uuid_representer(dumper, data):
@@ -123,6 +127,7 @@ def is_allowed_command(ctx):
     return ctx.invoked_subcommand in WARNING_UNPROTECTED_COMMANDS or "-h" in sys.argv or "--help" in sys.argv
 
 
+@with_plugins(iter_entry_points("renku.cli_plugins"))
 @click.group(
     cls=IssueFromTraceback, context_settings={"auto_envvar_prefix": "RENKU", "help_option_names": ["-h", "--help"],}
 )
@@ -189,10 +194,13 @@ cli.add_command(clone)
 cli.add_command(config)
 cli.add_command(dataset)
 cli.add_command(doctor)
-cli.add_command(githooks)
-cli.add_command(init)
+cli.add_command(githooks_command)
+cli.add_command(graph)
+cli.add_command(init_command)
 cli.add_command(log)
 cli.add_command(migrate)
+cli.add_command(migrationscheck)
+cli.add_command(check_immutable_template_files)
 cli.add_command(move)
 cli.add_command(remove)
 cli.add_command(rerun)
@@ -203,3 +211,4 @@ cli.add_command(status)
 cli.add_command(storage)
 cli.add_command(update)
 cli.add_command(workflow)
+cli.add_command(service)
