@@ -59,8 +59,9 @@ from urllib.parse import urlencode
 
 import click
 import filelock
+import portalocker
 
-from renku.core.errors import ParameterError, RenkuException, UsageError
+from renku.core.errors import MigrationRequired, ParameterError, ProjectNotSupported, RenkuException, UsageError
 
 _BUG = click.style("Ahhhhhhhh! You have found a bug. 🐞\n\n", fg="red", bold=True,)
 
@@ -86,12 +87,16 @@ class RenkuExceptionsHandler(click.Group):
         try:
             return super().main(*args, **kwargs)
         except RenkuException as e:
-            click.echo("Error: {}".format(e))
+            click.echo(f"Error: {e}", err=True)
             if e.__cause__ is not None:
-                click.echo("\n{}".format(traceback.format_exc()))
+                click.echo(f"\n{traceback.format_exc()}")
             exit_code = 1
             if isinstance(e, (ParameterError, UsageError)):
                 exit_code = 2
+            elif isinstance(e, MigrationRequired):
+                exit_code = 3
+            elif isinstance(e, ProjectNotSupported):
+                exit_code = 4
             sys.exit(exit_code)
 
 
@@ -109,7 +114,7 @@ class IssueFromTraceback(RenkuExceptionsHandler):
         if HAS_SENTRY:
             import sentry_sdk
 
-            sentry_sdk.init()
+            sentry_sdk.init(dsn=os.getenv("SENTRY_DSN"), environment=os.getenv("SENTRY_ENV"))
 
     def main(self, *args, **kwargs):
         """Catch all exceptions."""
@@ -117,7 +122,7 @@ class IssueFromTraceback(RenkuExceptionsHandler):
             result = super().main(*args, **kwargs)
             return result
 
-        except filelock.Timeout:
+        except (filelock.Timeout, portalocker.LockException, portalocker.AlreadyLocked):
             click.echo(
                 (
                     click.style("Unable to acquire lock.\n", fg="red",) + "Hint: Please wait for another renku "
@@ -143,7 +148,7 @@ class IssueFromTraceback(RenkuExceptionsHandler):
             with capture_internal_exceptions():
                 from git import Repo
 
-                from renku.core.commands import get_git_home
+                from renku.core.commands.git import get_git_home
                 from renku.core.models.provenance.agents import Person
 
                 repo = Repo(get_git_home())
