@@ -134,22 +134,16 @@ was not installed previously.
 
 import configparser
 import os
-from collections import OrderedDict, namedtuple
 from pathlib import Path
 from tempfile import mkdtemp
 
-import attr
 import click
-import pkg_resources
 from git import Repo
 
+from renku.cli.utils.callback import ClickCallback
 from renku.core import errors
-from renku.core.commands.client import pass_local_client
-from renku.core.commands.echo import INFO
-from renku.core.commands.git import set_git_home
-from renku.core.commands.init import create_from_template, fetch_template, read_template_manifest
+from renku.core.commands.init import init_command, is_path_empty
 from renku.core.commands.options import option_external_storage_requested
-from renku.core.models.tabulate import tabulate
 
 _GITLAB_CI = ".gitlab-ci.yml"
 _DOCKERFILE = "Dockerfile"
@@ -180,67 +174,6 @@ def validate_name(ctx, param, value):
     return value
 
 
-def store_directory(value):
-    """Store directory as a new Git home."""
-    Path(value).mkdir(parents=True, exist_ok=True)
-    set_git_home(value)
-    return value
-
-
-def create_template_sentence(templates, describe=False, instructions=False):
-    """Create templates choice sentence.
-
-    :ref templates: list of templates coming from manifest file
-    :ref instructions: add instructions
-    """
-    Template = namedtuple("Template", ["index", "id", "description", "variables"])
-
-    def extract_description(template_elem):
-        """Extract description from template manifest."""
-        if describe:
-            return template_elem["description"]
-        return None
-
-    def extract_variables(template_elem):
-        """Extract variables from tempalte manifest."""
-        if describe:
-            return "\n".join(
-                [f"{variable[0]}: {variable[1]}" for variable in template_elem.get("variables", {}).items()]
-            )
-
-        return ",".join(template_elem.get("variables", {}).keys())
-
-    templates_friendly = [
-        Template(
-            index=index + 1,
-            id=template_elem["folder"],
-            description=extract_description(template_elem),
-            variables=extract_variables(template_elem),
-        )
-        for index, template_elem in enumerate(templates)
-    ]
-
-    table_headers = OrderedDict((("index", "Index"), ("id", "Id"), ("variables", "Parameters"),))
-
-    if describe:
-        table_headers["description"] = "Description"
-
-    text = tabulate(templates_friendly, headers=table_headers)
-
-    if not instructions:
-        return text
-    return "{0}\nPlease choose a template by typing the index".format(text)
-
-
-def is_path_empty(path):
-    """Check if path contains files.
-
-    :ref path: target path
-    """
-    gen = Path(path).glob("**/*")
-    return not any(gen)
-
-
 def resolve_data_directory(data_dir, path):
     """Check data directory is within the project path."""
     if not data_dir:
@@ -296,6 +229,7 @@ def check_git_user_config():
 @click.option(
     "-p",
     "--parameter",
+    "metadata",
     multiple=True,
     type=click.STRING,
     callback=parse_parameters,
@@ -308,11 +242,9 @@ def check_git_user_config():
 @click.option("-d", "--describe", is_flag=True, help="Show description for templates and parameters")
 @click.option("--force", is_flag=True, help="Override target path.")
 @option_external_storage_requested
-@pass_local_client
 @click.pass_context
 def init(
     ctx,
-    client,
     external_storage_requested,
     path,
     name,
@@ -320,7 +252,7 @@ def init(
     template_index,
     template_source,
     template_ref,
-    parameter,
+    metadata,
     list_templates,
     force,
     describe,
@@ -329,9 +261,12 @@ def init(
     """Initialize a project in PATH. Default is the current path."""
     # verify dirty path
     if not is_path_empty(path) and not force and not list_templates:
+        existing_paths = [str(p.relative_to(path)) for p in Path(path).iterdir()]
+        existing_paths.sort()
         raise errors.InvalidFileOperation(
-            'Folder "{0}" is not empty. Please add --force '
-            "flag to transform it into a Renku repository.".format(str(path))
+            f'Folder "{str(path)}" is not empty and contains the following files/directories:'
+            + "".join((f"\n\t{e}" for e in existing_paths))
+            + "\nPlease add --force flag to transform it into a Renku repository."
         )
 
     data_dir = resolve_data_directory(data_dir, path)
@@ -344,121 +279,26 @@ def init(
             "\tgit config --global --add user.email "
             '"john.doe@example.com"\n'
         )
-
-    # select template source
-    if template_source:
-        click.echo("Fetching template from {0}@{1}... ".format(template_source, template_ref), nl=False)
-        template_folder = Path(mkdtemp())
-        fetch_template(template_source, template_ref, template_folder)
-        template_manifest = read_template_manifest(template_folder, checkout=True)
-        click.secho("OK", fg="green")
-    else:
-        template_folder = Path(pkg_resources.resource_filename("renku", "templates"))
-        template_manifest = read_template_manifest(template_folder)
-
-    # select specific template
-    repeat = False
-    template_data = None
-    if template_id:
-        if template_index:
-            raise errors.ParameterError("Use either --template-id or --template-index, not both", '"--template-index"')
-        template_filtered = [
-            template_elem for template_elem in template_manifest if template_elem["folder"] == template_id
-        ]
-        if len(template_filtered) == 1:
-            template_data = template_filtered[0]
-        else:
-            click.echo(f'The template with id "{template_id}" is not available.')
-            repeat = True
-
-    if template_index or template_index == 0:
-        if template_index > 0 and template_index <= len(template_manifest):
-            template_data = template_manifest[template_index - 1]
-        else:
-            click.echo(f"The template at index {template_index} is not available.")
-            repeat = True
+    communicator = ClickCallback()
+    init_command().with_communicator(communicator).build().execute(
+        ctx=ctx,
+        external_storage_requested=external_storage_requested,
+        path=path,
+        name=name,
+        template_id=template_id,
+        template_index=template_index,
+        template_source=template_source,
+        template_ref=template_ref,
+        metadata=metadata,
+        list_templates=list_templates,
+        force=force,
+        describe=describe,
+        data_dir=data_dir,
+    )
 
     if list_templates:
-        if template_data:
-            click.echo(create_template_sentence([template_data], describe=describe))
-        else:
-            click.echo(create_template_sentence(template_manifest, describe=describe))
         return
 
-    if repeat or not (template_id or template_index):
-        templates = [template_elem for template_elem in template_manifest]
-        if len(templates) == 1:
-            template_data = templates[0]
-        else:
-            template_num = click.prompt(
-                text=create_template_sentence(templates, describe=describe, instructions=True),
-                type=click.IntRange(1, len(templates)),
-                show_default=False,
-                show_choices=False,
-            )
-            template_data = templates[template_num - 1]
-
-    # verify variables have been passed
-    template_variables = template_data.get("variables", {})
-    template_variables_keys = set(template_variables.keys())
-    input_parameters_keys = set(parameter.keys())
-    for key in template_variables_keys - input_parameters_keys:
-        value = click.prompt(
-            text=(f'The template requires a value for "{key}" ' f"({template_variables[key]})"),
-            default="",
-            show_default=False,
-        )
-        parameter[key] = value
-    useless_variables = input_parameters_keys - template_variables_keys
-    if len(useless_variables) > 0:
-        click.echo(
-            INFO + "These parameters are not used by the template and were "
-            "ignored:\n\t{}".format("\n\t".join(useless_variables))
-        )
-        for key in useless_variables:
-            del parameter[key]
-
-    # set local path and storage
-    store_directory(path)
-    if not client.external_storage_requested:
-        external_storage_requested = False
-    ctx.obj = client = attr.evolve(
-        client, path=path, data_dir=data_dir, external_storage_requested=external_storage_requested
-    )
-    if not is_path_empty(path):
-        from git import GitCommandError
-
-        try:
-            commit = client.find_previous_commit("*")
-            branch_name = "pre_renku_init_{0}".format(commit.hexsha[:7])
-            with client.worktree(
-                path=path,
-                branch_name=branch_name,
-                commit=commit,
-                merge_args=["--no-ff", "-s", "recursive", "-X", "ours", "--allow-unrelated-histories"],
-            ):
-                click.echo("Saving current data in branch {0}".format(branch_name))
-        except AttributeError:
-            click.echo("Warning! Overwriting non-empty folder.")
-        except GitCommandError as e:
-            click.UsageError(e)
-
-    # clone the repo
-    template_path = template_folder / template_data["folder"]
-    click.echo("Initializing new Renku repository... ", nl=False)
-    with client.lock:
-        try:
-            create_from_template(
-                template_path=template_path,
-                client=client,
-                name=name,
-                metadata=parameter,
-                force=force,
-                data_dir=data_dir,
-            )
-        except FileExistsError as e:
-            raise click.UsageError(e)
-
     # Install git hooks
     from .githooks import install
 
