@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020- Swiss Data Science Center (SDSC)
+# Copyright 2018-2021- Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -95,7 +95,7 @@ class DAG(object):
         latest = self.graph.latest(node)
 
         # TODO move reference names to entity objects
-        from renku.cli.workflow import _deref
+        from renku.core.commands.workflow import _deref
 
         refs = node.client.workflow_names[node.path]
         formatted_refs = (
