@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -91,7 +91,7 @@ class ParameterError(RenkuException):
 
 
 class InvalidFileOperation(RenkuException):
-    """Raise when trying to perfrom invalid file operation."""
+    """Raise when trying to perform invalid file operation."""
 
 
 class UsageError(RenkuException):
@@ -196,6 +196,10 @@ class ProjectNotSupported(RenkuException):
         )
 
 
+class ProjectNotFound(RenkuException):
+    """Raise when one or more projects couldn't be found in the KG."""
+
+
 class NothingToCommit(RenkuException):
     """Raise when there is nothing to commit."""
 
@@ -382,6 +386,10 @@ class GitError(RenkuException):
     """Raised when a remote Git repo cannot be accessed."""
 
 
+class GitLFSError(RenkuException):
+    """Raised when a Git LFS operation fails."""
+
+
 class UrlSchemeNotSupported(RenkuException):
     """Raised when adding data from unsupported URL schemes."""
 
@@ -419,6 +427,18 @@ class ExportError(RenkuException):
     """Raised when a dataset cannot be exported."""
 
 
+class TemplateUpdateError(RenkuException):
+    """Raised when a project couldn't be updated from its template."""
+
+
+class DockerfileUpdateError(RenkuException):
+    """Raised when the renku version in the Dockerfile couldn't be updated."""
+
+
+class MigrationError(RenkuException):
+    """Raised when something went wrong during migrations."""
+
+
 class RenkuImportError(RenkuException):
     """Raised when a dataset cannot be imported."""
 
@@ -426,3 +446,19 @@ class RenkuImportError(RenkuException):
         """Embed exception and build a custom message."""
         self.exp = exp
         super(RenkuImportError, self).__init__(msg)
+
+
+class CommandNotFinalizedError(RenkuException):
+    """Raised when a non-finalized command is executed."""
+
+
+class CommandFinalizedError(RenkuException):
+    """Raised when trying to modify a finalized command builder."""
+
+
+class RenkuSaveError(RenkuException):
+    """Raised when renku save doesn't work."""
+
+
+class DatasetImageError(RenkuException):
+    """Raised when a local dataset image is not accessible."""
