@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -18,8 +18,10 @@
 """Client for handling datasets."""
 
 import concurrent.futures
+import imghdr
 import os
 import re
+import shlex
 import shutil
 import tempfile
 import time
@@ -27,9 +29,10 @@ import uuid
 from collections import OrderedDict
 from contextlib import contextmanager
 from pathlib import Path
-from subprocess import PIPE, SubprocessError, run
+from subprocess import PIPE, SubprocessError
 from urllib import error, parse
-from urllib.parse import ParseResult
+from urllib.parse import ParseResult, urlparse
+from urllib.request import urlretrieve
 
 import attr
 import patoolib
@@ -45,13 +48,18 @@ from renku.core.models.datasets import (
     Dataset,
     DatasetFile,
     DatasetTag,
+    ImageObject,
     generate_dataset_file_url,
+    get_slug,
     is_dataset_name_valid,
 )
 from renku.core.models.git import GitURL
-from renku.core.models.locals import with_reference
 from renku.core.models.provenance.agents import Person
+from renku.core.models.provenance.datasets import DatasetProvenance
 from renku.core.models.refs import LinkReference
+from renku.core.utils import communication
+from renku.core.utils.git import add_to_git, run_command
+from renku.core.utils.migrate import MigrationType
 from renku.core.utils.urls import remove_credentials
 
 
@@ -68,11 +76,48 @@ class DatasetsApiMixin(object):
     CACHE = "cache"
     """Directory to cache transient data."""
 
+    DATASET_IMAGES = "dataset_images"
+    """Directory for dataset images."""
+
+    DATASETS_PROVENANCE = "dataset.json"
+    """File for storing datasets' provenance."""
+
+    _temporary_datasets_path = None
+    _datasets_provenance = None
+
     @property
     def renku_datasets_path(self):
         """Return a ``Path`` instance of Renku dataset metadata folder."""
+        if self._temporary_datasets_path:
+            return self._temporary_datasets_path
+
         return self.path / self.renku_home / self.DATASETS
 
+    @property
+    def renku_dataset_images_path(self):
+        """Return a ``Path`` instance of Renku dataset metadata folder."""
+        if self._temporary_datasets_path:
+            return self._temporary_datasets_path
+
+        return self.path / self.renku_home / self.DATASET_IMAGES
+
+    @property
+    def datasets_provenance_path(self):
+        """Path to store activity files."""
+        return self.renku_path / self.DATASETS_PROVENANCE
+
+    def set_temporary_datasets_path(self, path):
+        """Set path to Renku dataset metadata directory."""
+        self._temporary_datasets_path = path
+
+    def clear_temporary_datasets_path(self):
+        """Clear path to Renku dataset metadata directory."""
+        self._temporary_datasets_path = None
+
+    def is_using_temporary_datasets_path(self):
+        """Return true if temporary datasets path is set."""
+        return bool(self._temporary_datasets_path)
+
     @property
     def renku_pointers_path(self):
         """Return a ``Path`` instance of Renku pointer files folder."""
@@ -80,6 +125,43 @@ class DatasetsApiMixin(object):
         path.mkdir(exist_ok=True)
         return path
 
+    @property
+    def datasets_provenance(self):
+        """Return dataset provenance if available."""
+        if not self.has_datasets_provenance():
+            return
+        if not self._datasets_provenance:
+            self._datasets_provenance = DatasetProvenance.from_json(self.datasets_provenance_path)
+
+        return self._datasets_provenance
+
+    def update_datasets_provenance(self, dataset, remove=False):
+        """Update datasets provenance for a dataset."""
+        if not self.has_datasets_provenance():
+            return
+
+        if remove:
+            self.datasets_provenance.remove_dataset(dataset=dataset, client=self)
+        else:
+            self.datasets_provenance.update_dataset(dataset=dataset, client=self)
+
+        self.datasets_provenance.to_json()
+
+    def has_datasets_provenance(self):
+        """Return true if dataset provenance exists."""
+        return self.datasets_provenance_path.exists()
+
+    def remove_datasets_provenance_file(self):
+        """Remove dataset provenance."""
+        try:
+            self.datasets_provenance_path.unlink()
+        except FileNotFoundError:
+            pass
+
+    def initialize_datasets_provenance(self):
+        """Create empty dataset provenance file."""
+        self.datasets_provenance_path.write_text("[]")
+
     def datasets_from_commit(self, commit=None):
         """Return datasets defined in a commit."""
         commit = commit or self.repo.head.commit
@@ -102,11 +184,14 @@ class DatasetsApiMixin(object):
     def datasets(self):
         """Return mapping from path to dataset."""
         result = {}
-        paths = self.renku_datasets_path.rglob(self.METADATA)
-        for path in paths:
+        for path in self.get_datasets_metadata_files():
             result[path] = self.load_dataset_from_path(path)
         return result
 
+    def get_datasets_metadata_files(self):
+        """Return a generator of datasets metadata files."""
+        return self.renku_datasets_path.rglob(self.METADATA)
+
     def load_dataset_from_path(self, path, commit=None):
         """Return a dataset from a given path."""
         path = Path(path)
@@ -125,18 +210,26 @@ class DatasetsApiMixin(object):
 
         return path
 
-    def load_dataset(self, name=None):
+    def load_dataset(self, name=None, strict=False):
         """Load dataset reference file."""
+        dataset = None
         if name:
             path = self.get_dataset_path(name)
             if path and path.exists():
-                return self.load_dataset_from_path(path)
+                dataset = self.load_dataset_from_path(path)
+
+        if not dataset and strict:
+            raise errors.DatasetNotFound(name=name)
+
+        return dataset
 
     @contextmanager
-    def with_dataset(self, name=None, create=False):
+    def with_dataset(self, name=None, create=False, immutable=False):
         """Yield an editable metadata object for a dataset."""
         dataset = self.load_dataset(name=name)
         clean_up_required = False
+        dataset_ref = None
+        path = None
 
         if dataset is None:
             if not create:
@@ -146,6 +239,8 @@ class DatasetsApiMixin(object):
             dataset, path, dataset_ref = self.create_dataset(name=name)
         elif create:
             raise errors.DatasetExistsError('Dataset exists: "{}".'.format(name))
+
+        dataset.immutable = immutable or create
         dataset_path = self.path / self.data_dir / dataset.name
         dataset_path.mkdir(parents=True, exist_ok=True)
 
@@ -161,15 +256,61 @@ class DatasetsApiMixin(object):
 
         dataset.to_yaml()
 
+    @contextmanager
+    def with_dataset_provenance(self, name=None, create=False):
+        """Yield a dataset's metadata from dataset provenance."""
+        dataset = self.load_dataset_from_provenance(name=name)
+        clean_up_required = False
+        dataset_ref = None
+        path = None
+
+        if dataset is None:
+            if not create:
+                raise errors.DatasetNotFound(name=name)
+
+            clean_up_required = True
+            dataset, path, dataset_ref = self.create_dataset(name=name)
+        elif create:
+            raise errors.DatasetExistsError('Dataset exists: "{}".'.format(name))
+        else:
+            dataset = dataset.to_dataset(self)
+
+        dataset_path = self.path / self.data_dir / dataset.name
+        dataset_path.mkdir(parents=True, exist_ok=True)
+
+        try:
+            yield dataset
+        except Exception:
+            # TODO use a general clean-up strategy
+            # https://github.com/SwissDataScienceCenter/renku-python/issues/736
+            if clean_up_required:
+                dataset_ref.delete()
+                shutil.rmtree(path.parent, ignore_errors=True)
+            raise
+
+        dataset.to_yaml(os.path.join(self.path, dataset.path, self.METADATA))
+
+    def load_dataset_from_provenance(self, name, strict=False):
+        """Load latest dataset's metadata from dataset provenance file."""
+        dataset = None
+        if name:
+            dataset = self.datasets_provenance.get_latest_by_name(name)
+
+        if not dataset and strict:
+            raise errors.DatasetNotFound(name=name)
+
+        return dataset
+
     def create_dataset(
-        self, name=None, title=None, description=None, creators=None, keywords=None,
+        self, name=None, title=None, description=None, creators=None, keywords=None, images=None, safe_image_paths=[]
     ):
         """Create a dataset."""
         if not name:
             raise errors.ParameterError("Dataset name must be provided.")
 
         if not is_dataset_name_valid(name):
-            raise errors.ParameterError('Dataset name "{}" is not valid.'.format(name))
+            valid_name = get_slug(name)
+            raise errors.ParameterError(f'Dataset name "{name}" is not valid (Hint: "{valid_name}" is valid).')
 
         if self.load_dataset(name=name):
             raise errors.DatasetExistsError('Dataset exists: "{}".'.format(name))
@@ -179,36 +320,136 @@ class DatasetsApiMixin(object):
 
         identifier = str(uuid.uuid4())
 
-        path = self.renku_datasets_path / identifier / self.METADATA
+        metadata_path = self.renku_datasets_path / identifier / self.METADATA
 
-        if path.exists():
-            raise errors.DatasetExistsError("Dataset with reference {} exists".format(path))
+        if metadata_path.exists():
+            raise errors.DatasetExistsError(f"Dataset with reference {metadata_path} exists")
 
-        path.parent.mkdir(parents=True, exist_ok=True)
+        dataset_path = metadata_path.parent
+        dataset_path.mkdir(exist_ok=True, parents=True)
+        dataset_path = dataset_path.relative_to(self.path)
 
         if creators is None:
             creators = [Person.from_git(self.repo)]
 
         keywords = keywords or ()
 
-        with with_reference(path):
-            dataset = Dataset(
-                client=self,
-                identifier=identifier,
-                name=name,
-                title=title,
-                description=description,
-                creators=creators,
-                keywords=keywords,
-            )
+        dataset = Dataset(
+            client=self,
+            identifier=identifier,
+            name=name,
+            title=title,
+            path=dataset_path,
+            description=description,
+            creators=creators,
+            keywords=keywords,
+            immutable=True,  # No mutation required when first creating a dataset
+        )
+
+        if images:
+            safe_image_paths.append(self.path)
+            self.set_dataset_images(dataset, images, safe_image_paths)
 
         dataset_ref = LinkReference.create(client=self, name="datasets/" + name)
+        dataset_ref.set_reference(metadata_path)
 
-        dataset_ref.set_reference(path)
-        dataset.path = Path(dataset.path).relative_to(self.path)
-        dataset.to_yaml()
+        dataset.to_yaml(path=metadata_path)
 
-        return dataset, path, dataset_ref
+        return dataset, metadata_path, dataset_ref
+
+    def set_dataset_images(self, dataset, images, safe_image_paths=[]):
+        """Set the images on a dataset."""
+
+        if not images:
+            images = []
+
+        (self.renku_dataset_images_path / dataset.identifier).mkdir(exist_ok=True, parents=True)
+
+        previous_images = dataset.images or []
+
+        dataset.images = []
+
+        images_updated = False
+
+        for img in images:
+            position = img["position"]
+            content_url = img["content_url"]
+
+            if any(i.position == img["position"] for i in dataset.images):
+                raise errors.DatasetImageError(f"Duplicate dataset image specified for position {position}")
+
+            existing = next(
+                (i for i in previous_images if i.position == img["position"] and i.content_url == img["content_url"]),
+                None,
+            )
+
+            if existing:
+                dataset.images.append(existing)
+                continue
+            image_type = None
+            if urlparse(content_url).netloc:
+                # NOTE: absolute url
+                if not img.get("mirror_locally", False):
+                    dataset.images.append(
+                        ImageObject(content_url, position, id=ImageObject.generate_id(dataset, position))
+                    )
+                    images_updated = True
+                    continue
+
+                # NOTE: mirror the image locally
+                try:
+                    path, _ = urlretrieve(content_url)
+                except error.URLError as e:
+                    raise errors.DatasetImageError(f"Dataset image with url {content_url} couldn't be mirrored") from e
+
+                image_type = imghdr.what(path)
+                if image_type:
+                    image_type = f".{image_type}"
+
+                content_url = path
+                safe_image_paths.append(Path(path).parent)
+
+            path = content_url
+            if not os.path.isabs(path):
+                path = os.path.normpath(os.path.join(self.path, path))
+
+            if not os.path.exists(path) or not any(os.path.commonprefix([path, p]) == str(p) for p in safe_image_paths):
+                # NOTE: make sure files exists and prevent path traversal
+                raise errors.DatasetImageError(f"Dataset image with relative path {content_url} not found")
+
+            image_folder = self.renku_dataset_images_path / dataset.identifier
+
+            if not path.startswith(str(image_folder)):
+                # NOTE: only copy dataset image if it's not in .renku/datasets/<id>/images/ already
+                if image_type:
+                    ext = image_type
+                else:
+                    _, ext = os.path.splitext(content_url)
+
+                img_path = image_folder / f"{position}{ext}"
+                shutil.copy(path, img_path)
+
+            dataset.images.append(
+                ImageObject(
+                    str(img_path.relative_to(self.path)), position, id=ImageObject.generate_id(dataset, position)
+                )
+            )
+            images_updated = True
+
+        new_urls = [i.content_url for i in dataset.images]
+
+        for prev in previous_images:
+            # NOTE: Delete images if they were removed
+            if prev.content_url in new_urls or urlparse(prev.content_url).netloc:
+                continue
+
+            path = prev.content_url
+            if not os.path.isabs(path):
+                path = os.path.normpath(os.path.join(self.path, path))
+
+            os.remove(path)
+
+        return images_updated or dataset.images != previous_images
 
     def add_data_to_dataset(
         self,
@@ -223,11 +464,8 @@ class DatasetsApiMixin(object):
         extract=False,
         all_at_once=False,
         destination_names=None,
-        progress=None,
     ):
         """Import the data into the data directory."""
-        messages = []
-        warning_messages = []
         dataset_datadir = self.path / dataset.data_dir
 
         destination = destination or Path(".")
@@ -242,12 +480,7 @@ class DatasetsApiMixin(object):
         files = []
         if all_at_once:  # Importing a dataset
             files = self._add_from_urls(
-                dataset=dataset,
-                urls=urls,
-                destination_names=destination_names,
-                destination=destination,
-                extract=extract,
-                progress=progress,
+                urls=urls, destination_names=destination_names, destination=destination, extract=extract,
             )
         else:
             for url in urls:
@@ -255,16 +488,14 @@ class DatasetsApiMixin(object):
 
                 if is_git and is_remote:  # Remote git repo
                     sources = sources or ()
-                    new_files = self._add_from_git(
-                        dataset=dataset, url=url, sources=sources, destination=destination, ref=ref
-                    )
+                    new_files = self._add_from_git(url=url, sources=sources, destination=destination, ref=ref)
                 else:
                     if sources:
                         raise errors.UsageError('Cannot use "--source" with URLs or local files.')
 
                     if not is_remote:  # Local path, might be git
                         if is_git:
-                            warning_messages.append(
+                            communication.warn(
                                 "Adding data from local Git repository: "
                                 + "Use remote's Git URL instead to enable "
                                 + "lineage information and updates."
@@ -274,9 +505,7 @@ class DatasetsApiMixin(object):
                             dataset=dataset, path=u.path, external=external, destination=destination
                         )
                     else:  # Remote URL
-                        new_files = self._add_from_url(
-                            dataset=dataset, url=url, destination=destination, extract=extract, progress=progress
-                        )
+                        new_files = self._add_from_url(url=url, destination=destination, extract=extract)
 
                 files.extend(new_files)
 
@@ -284,7 +513,7 @@ class DatasetsApiMixin(object):
         paths_to_avoid = [f["path"] for f in files if ".git" in str(f["path"]).split(os.path.sep)]
         if paths_to_avoid:
             files = [f for f in files if f["path"] not in paths_to_avoid]
-            warning_messages.append(
+            communication.warn(
                 "Ignored adding paths under a .git directory:\n  " + "\n  ".join(str(p) for p in paths_to_avoid)
             )
 
@@ -306,7 +535,7 @@ class DatasetsApiMixin(object):
                             ignored_sources.append(file_["path"])
 
                 files = [f for f in files if str(self.path / f["path"]) in files_to_commit]
-                warning_messages.append(
+                communication.warn(
                     "Theses paths are ignored by one of your .gitignore "
                     + 'files (use "--force" flag if you really want to add '
                     + "them):\n  "
@@ -320,7 +549,7 @@ class DatasetsApiMixin(object):
             if existing_files:
                 files_to_commit = files_to_commit.difference(existing_files)
                 files = [f for f in files if str(self.path / f["path"]) in files_to_commit]
-                warning_messages.append(
+                communication.warn(
                     "These existing files were not overwritten "
                     + '(use "--overwrite" flag to overwrite them):\n  '
                     + "\n  ".join([str(p) for p in existing_files])
@@ -352,17 +581,17 @@ class DatasetsApiMixin(object):
             lfs_paths = self.track_paths_in_storage(*files_to_commit)
             show_message = self.get_value("renku", "show_lfs_message")
             if lfs_paths and (show_message is None or show_message == "True"):
-                messages.append(
+                communication.info(
                     (
                         "Adding these files to Git LFS:\n"
                         + "\t{}".format("\n\t".join(lfs_paths))
                         + "\nTo disable this message in the future, run:"
-                        + "\n\trenku config show_lfs_message False"
+                        + "\n\trenku config set show_lfs_message False"
                     )
                 )
 
         # Force-add to include possible ignored files
-        self.repo.git.add(*files_to_commit, force=True)
+        add_to_git(self.repo.git, *files_to_commit, force=True)
         self.repo.git.add(self.renku_pointers_path, force=True)
 
         staged_files = self.repo.index.diff("HEAD")
@@ -371,7 +600,7 @@ class DatasetsApiMixin(object):
             skip_hooks = not self.external_storage_requested
             self.repo.index.commit(msg, skip_hooks=skip_hooks)
         else:
-            warning_messages.append("No file was added to project")
+            communication.warn("No new file was added to project")
 
         # Generate the DatasetFiles
         dataset_files = []
@@ -385,7 +614,6 @@ class DatasetsApiMixin(object):
             dataset_files.append(dataset_file)
 
         dataset.update_files(dataset_files)
-        return warning_messages, messages
 
     def _check_protected_path(self, path):
         """Checks if a path is a protected path."""
@@ -453,19 +681,29 @@ class DatasetsApiMixin(object):
             }
         ]
 
-    def _add_from_urls(self, dataset, urls, destination, destination_names, extract, progress):
+    def _add_from_urls(self, urls, destination, destination_names, extract):
+        listeners = communication.get_listeners()
+
+        def subscribe_communication_listeners(function, **kwargs):
+            try:
+                for communicator in listeners:
+                    communication.subscribe(communicator)
+                return function(**kwargs)
+            finally:
+                for communicator in listeners:
+                    communication.unsubscribe(communicator)
+
         files = []
         max_workers = min(os.cpu_count() - 1, 4) or 1
         with concurrent.futures.ThreadPoolExecutor(max_workers) as executor:
             futures = {
                 executor.submit(
+                    subscribe_communication_listeners,
                     self._add_from_url,
-                    dataset=dataset,
                     url=url,
                     destination=destination,
                     extract=extract,
                     filename=name,
-                    progress=progress,
                 )
                 for url, name in zip(urls, destination_names)
             }
@@ -475,13 +713,13 @@ class DatasetsApiMixin(object):
 
         return files
 
-    def _add_from_url(self, dataset, url, destination, extract, filename=None, progress=None):
+    def _add_from_url(self, url, destination, extract, filename=None):
         """Process adding from url and return the location on disk."""
         url = self._provider_check(url)
 
         try:
             start = time.time() * 1e3
-            tmp_root, paths = self._download(url=url, filename=filename, extract=extract, progress_class=progress)
+            tmp_root, paths = self._download(url=url, filename=filename, extract=extract)
 
             exec_time = (time.time() * 1e3 - start) // 1e3
             # If execution time was less or equal to zero seconds,
@@ -503,7 +741,7 @@ class DatasetsApiMixin(object):
             for src, dst in paths
         ]
 
-    def _add_from_git(self, dataset, url, sources, destination, ref):
+    def _add_from_git(self, url, sources, destination, ref):
         """Process adding resources from another git repository."""
         from renku import LocalClient
 
@@ -571,7 +809,7 @@ class DatasetsApiMixin(object):
                 if remote_client._is_external_file(src):
                     operation = (src.resolve(), dst, "symlink")
                 else:
-                    operation = (src, dst, "copy")
+                    operation = (src, dst, "move")
 
                 results.append(
                     {
@@ -677,23 +915,24 @@ class DatasetsApiMixin(object):
     def _fetch_lfs_files(repo_path, paths):
         """Fetch and checkout paths that are tracked by Git LFS."""
         repo_path = str(repo_path)
-        try:
-            output = run(("git", "lfs", "ls-files", "--name-only"), stdout=PIPE, cwd=repo_path, universal_newlines=True)
-        except SubprocessError:
-            return
-
-        lfs_files = set(output.stdout.split("\n"))
-        files = lfs_files & paths
-        if not files:
-            return
 
         try:
-            for path in files:
-                run(["git", "lfs", "pull", "--include", path], cwd=repo_path)
+            paths = [shlex.quote(p) for p in paths]
+            status = run_command(
+                ["git", "lfs", "pull", "--include"],
+                *paths,
+                separator=",",
+                stderr=PIPE,
+                cwd=repo_path,
+                universal_newlines=True,
+            )
+            if status.returncode != 0:
+                message = "\n\t".join(status.stderr.split("\n"))
+                raise errors.GitError(f"Cannot pull LFS objects from server: {message}")
         except KeyboardInterrupt:
             raise
-        except SubprocessError:
-            pass
+        except SubprocessError as e:
+            raise errors.GitError(f"Cannot pull LFS objects from server: {e}")
 
     @staticmethod
     def _fetch_files_metadata(client, paths):
@@ -703,7 +942,13 @@ class DatasetsApiMixin(object):
         if is_project_unsupported(client):
             return files
 
-        migrate(client)
+        migration_type = client.migration_type
+        # NOTE: We are not interested in migrating workflows when looking for dataset metadata
+        client.migration_type = ~MigrationType.WORKFLOWS
+        try:
+            migrate(client, skip_template_update=True, skip_docker_update=True)
+        finally:
+            client.migration_type = migration_type
 
         for _, dataset in client.datasets.items():
             for file_ in dataset.files:
@@ -760,8 +1005,6 @@ class DatasetsApiMixin(object):
 
         dataset.tags.append(tag)
 
-        return dataset
-
     def remove_dataset_tags(self, dataset, tags):
         """Removes tags from a dataset."""
         tag_names = {t.name for t in dataset.tags}
@@ -771,9 +1014,7 @@ class DatasetsApiMixin(object):
             raise errors.ParameterError("Tags {} not found".format(", ".join(not_found)))
         dataset.tags = [t for t in dataset.tags if t.name not in tags]
 
-        return dataset
-
-    def update_dataset_files(self, files, ref, delete=False):
+    def update_dataset_git_files(self, files, ref, delete=False):
         """Update files and dataset metadata according to their remotes.
 
         :param files: List of files to be updated
@@ -787,62 +1028,70 @@ class DatasetsApiMixin(object):
         updated_files = []
         deleted_files = []
 
-        for file_ in files:
-            if not file_.based_on:
-                continue
+        progress_text = "Checking files for updates"
 
-            file_.based_on = DatasetFile.from_jsonld(file_.based_on)
-            based_on = file_.based_on
-            url = based_on.source
-            if url in visited_repos:
-                repo, repo_path, remote_client = visited_repos[url]
-            else:
-                repo, repo_path = self.prepare_git_repo(url, ref)
-                remote_client = LocalClient(repo_path)
-                visited_repos[url] = repo, repo_path, remote_client
+        try:
+            communication.start_progress(progress_text, len(files))
+            for file_ in files:
+                communication.update_progress(progress_text, 1)
+                if not file_.based_on:
+                    continue
 
-            remote_file = self._fetch_file_metadata(remote_client, based_on.path)
-
-            if not remote_file:
-                try:
-                    remote_file = DatasetFile.from_revision(
-                        remote_client, path=based_on.path, source=None, added=based_on.added
-                    )
-                except KeyError:
-                    raise errors.ParameterError("Cannot find file {} in the repo {}".format(based_on.source, url))
-
-            commit_sha = self._get_commit_sha_from_label(based_on)
-            remote_commit_sha = self._get_commit_sha_from_label(remote_file)
-            if commit_sha != remote_commit_sha:
-                src = Path(repo.working_dir) / based_on.path
-                dst = self.renku_path.parent / file_.path
-
-                if src.exists():
-                    # Fetch file is it is tracked by Git LFS
-                    self._fetch_lfs_files(repo_path, {based_on.path})
-                    if remote_client._is_external_file(src):
-                        self.remove_file(dst)
-                        self._create_external_file(src.resolve(), dst)
-                    else:
-                        shutil.copy(src, dst)
-                    file_.based_on.commit = remote_file.commit
-                    file_.based_on._label = remote_file._label
-                    updated_files.append(file_)
+                file_.based_on = DatasetFile.from_jsonld(file_.based_on)
+                based_on = file_.based_on
+                url = based_on.source
+                if url in visited_repos:
+                    repo, repo_path, remote_client = visited_repos[url]
                 else:
-                    # File was removed or renamed
-                    if delete:
-                        self.remove_file(dst)
-                    deleted_files.append(file_)
+                    repo, repo_path = self.prepare_git_repo(url, ref)
+                    remote_client = LocalClient(repo_path)
+                    visited_repos[url] = repo, repo_path, remote_client
+
+                remote_file = self._fetch_file_metadata(remote_client, based_on.path)
+
+                if not remote_file:
+                    try:
+                        remote_file = DatasetFile.from_revision(
+                            remote_client, path=based_on.path, source=None, added=based_on.added
+                        )
+                    except KeyError:
+                        raise errors.ParameterError("Cannot find file {} in the repo {}".format(based_on.source, url))
+
+                commit_sha = self._get_commit_sha_from_label(based_on)
+                remote_commit_sha = self._get_commit_sha_from_label(remote_file)
+                if commit_sha != remote_commit_sha:
+                    src = Path(repo.working_dir) / based_on.path
+                    dst = self.renku_path.parent / file_.path
+
+                    if src.exists():
+                        # Fetch file if it is tracked by Git LFS
+                        self._fetch_lfs_files(repo_path, {based_on.path})
+                        if remote_client._is_external_file(src):
+                            self.remove_file(dst)
+                            self._create_external_file(src.resolve(), dst)
+                        else:
+                            shutil.copy(src, dst)
+                        file_.based_on.commit = remote_file.commit
+                        file_.based_on._label = file_.based_on.default_label()
+                        file_.based_on._id = file_.based_on.default_id()
+                        updated_files.append(file_)
+                    else:
+                        # File was removed or renamed
+                        if delete:
+                            self.remove_file(dst)
+                        deleted_files.append(file_)
+        finally:
+            communication.finalize_progress(progress_text)
 
         if not updated_files and (not delete or not deleted_files):
             # Nothing to commit or update
-            return deleted_files
+            return [], deleted_files
 
         # Commit changes in files
 
         file_paths = {str(self.path / f.path) for f in updated_files + deleted_files}
         # Force-add to include possible ignored files that are in datasets
-        self.repo.git.add(*(file_paths), force=True)
+        add_to_git(self.repo.git, *file_paths, force=True)
         skip_hooks = not self.external_storage_requested
         self.repo.index.commit(
             "renku dataset: updated {} files and deleted {} files".format(len(updated_files), len(deleted_files)),
@@ -854,7 +1103,9 @@ class DatasetsApiMixin(object):
         modified_datasets = {}
 
         for file_ in updated_files:
-            new_file = DatasetFile.from_revision(self, path=file_.path, based_on=file_.based_on, url=file_.url)
+            new_file = DatasetFile.from_revision(
+                self, path=file_.path, based_on=file_.based_on, url=file_.url, source=file_.source
+            )
             file_.dataset.update_files([new_file])
             modified_datasets[file_.dataset.name] = file_.dataset
 
@@ -865,8 +1116,9 @@ class DatasetsApiMixin(object):
 
         for dataset in modified_datasets.values():
             dataset.to_yaml()
+            self.update_datasets_provenance(dataset)
 
-        return deleted_files
+        return updated_files, deleted_files
 
     def _create_external_file(self, src, dst):
         """Create a new external file."""
@@ -924,16 +1176,17 @@ class DatasetsApiMixin(object):
         if not updated_files_paths:
             return
 
-        self.repo.git.add(*updated_files_paths, force=True)
+        add_to_git(self.repo.git, *updated_files_paths, force=True)
         self.repo.git.add(self.renku_pointers_path, force=True)
         commit = self.repo.index.commit("renku dataset: updated {} external files".format(len(updated_files_paths)))
 
         for dataset in updated_datasets.values():
             for file_ in dataset.files:
                 if str(self.path / file_.path) in updated_files_paths:
-                    file_.commit = commit
-                    file_._label = file_.default_label()
+                    file_.update_commit(commit)
+            dataset.mutate()
             dataset.to_yaml()
+            self.update_datasets_provenance(dataset)
 
     def _update_pointer_file(self, pointer_file_path):
         """Update a pointer file."""
@@ -952,7 +1205,8 @@ class DatasetsApiMixin(object):
         os.remove(pointer_file_path)
         return self._create_pointer_file(target, checksum=checksum)
 
-    def remove_file(self, filepath):
+    @staticmethod
+    def remove_file(filepath):
         """Remove a file/symlink and its pointer file (for external files)."""
         path = Path(filepath)
         try:
@@ -1000,7 +1254,7 @@ class DatasetsApiMixin(object):
         if not url:
             raise errors.GitError("Invalid URL.")
 
-        RENKU_BRANCH = "renku-default-branch"
+        renku_branch = "renku-default-branch"
 
         def checkout(repo, ref):
             try:
@@ -1008,7 +1262,8 @@ class DatasetsApiMixin(object):
             except GitCommandError:
                 raise errors.ParameterError('Cannot find reference "{}" in Git repository: {}'.format(ref, url))
 
-        ref = ref or RENKU_BRANCH
+        depth = 1 if not ref else None
+        ref = ref or renku_branch
         u = GitURL.parse(url)
         path = u.pathname
         if u.hostname == "localhost":
@@ -1022,6 +1277,7 @@ class DatasetsApiMixin(object):
             repo = Repo(str(repo_path))
             if repo.remotes.origin.url == url:
                 try:
+                    repo.git.checkout(".")
                     repo.git.fetch(all=True)
                     repo.git.checkout(ref)
                     try:
@@ -1040,12 +1296,12 @@ class DatasetsApiMixin(object):
             except PermissionError:
                 raise errors.InvalidFileOperation("Cannot delete files in {}: Permission denied".format(repo_path))
 
-        repo = clone(url, path=str(repo_path), install_githooks=False)
+        repo, _ = clone(url, path=str(repo_path), install_githooks=False, depth=depth)
 
         # Because the name of the default branch is not always 'master', we
         # create an alias of the default branch when cloning the repo. It
         # is used to refer to the default branch later.
-        renku_ref = "refs/heads/" + RENKU_BRANCH
+        renku_ref = "refs/heads/" + renku_branch
         try:
             repo.git.execute(["git", "symbolic-ref", renku_ref, repo.head.reference.path])
             checkout(repo, ref)
@@ -1066,10 +1322,10 @@ class DatasetsApiMixin(object):
     def _get_commit_sha_from_label(dataset_file):
         label = dataset_file._label
         if "@" in label:
-            return label.split("@")[1]
+            return label.rsplit("@", maxsplit=1)[-1]
         return label
 
-    def _download(self, url, filename, extract, progress_class=None, chunk_size=16384):
+    def _download(self, url, filename, extract, chunk_size=16384):
         def extract_dataset(filepath):
             """Extract downloaded file."""
             try:
@@ -1097,35 +1353,21 @@ class DatasetsApiMixin(object):
             download_to = Path(tmp) / filename
             with open(str(download_to), "wb") as file_:
                 total_size = int(request.headers.get("content-length", 0))
-                progress_class = progress_class or DownloadProgressCallback
-                progress = progress_class(description=filename, total_size=total_size)
 
+                communication.start_progress(name=filename, total=total_size)
                 try:
                     for chunk in request.iter_content(chunk_size=chunk_size):
                         if chunk:  # ignore keep-alive chunks
                             file_.write(chunk)
-                            progress.update(size=len(chunk))
+                            communication.update_progress(name=filename, amount=len(chunk))
                 finally:
-                    progress.finalize()
+                    communication.finalize_progress(name=filename)
         if extract:
             return extract_dataset(download_to)
 
         return download_to.parent, [download_to]
 
 
-class DownloadProgressCallback:
-    """Interface to report various stages of a download."""
-
-    def __init__(self, description, total_size):
-        """Default initializer."""
-
-    def update(self, size):
-        """Update the status."""
-
-    def finalize(self):
-        """Called once when the download is finished."""
-
-
 def _check_url(url):
     """Check if a url is local/remote and if it contains a git repository."""
     u = parse.urlparse(url)
@@ -1154,8 +1396,10 @@ def _check_url(url):
 
 
 DATASET_METADATA_PATHS = [
-    Path(RENKU_HOME) / Path(DatasetsApiMixin.DATASETS),
-    Path(RENKU_HOME) / Path(DatasetsApiMixin.POINTERS),
-    Path(RENKU_HOME) / Path(LinkReference.REFS),
+    Path(RENKU_HOME) / DatasetsApiMixin.DATASETS,
+    Path(RENKU_HOME) / DatasetsApiMixin.DATASET_IMAGES,
+    Path(RENKU_HOME) / DatasetsApiMixin.POINTERS,
+    Path(RENKU_HOME) / LinkReference.REFS,
+    Path(RENKU_HOME) / DatasetsApiMixin.DATASETS_PROVENANCE,
     ".gitattributes",
 ]
