@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -33,6 +33,8 @@ import attr
 import git
 
 from renku.core import errors
+from renku.core.utils.git import split_paths
+from renku.core.utils.scm import git_unicode_unescape, shorten_message
 from renku.core.utils.urls import remove_credentials
 
 COMMIT_DIFF_STRATEGY = "DIFF"
@@ -127,7 +129,10 @@ class GitCore:
     def dirty_paths(self):
         """Get paths of dirty files in the repository."""
         repo_path = self.repo.working_dir
-        return {os.path.join(repo_path, p) for p in self.repo.untracked_files + self.modified_paths}
+        staged_files = (
+            [git_unicode_unescape(d.a_path) for d in self.repo.index.diff("HEAD")] if self.repo.head.is_valid() else []
+        )
+        return {os.path.join(repo_path, p) for p in self.repo.untracked_files + self.modified_paths + staged_files}
 
     @property
     def candidate_paths(self):
@@ -142,23 +147,26 @@ class GitCore:
         """Return ignored paths matching ``.gitignore`` file."""
         from git.exc import GitCommandError
 
-        try:
-            return self.repo.git.check_ignore(*paths).split()
-        except GitCommandError:
-            pass
+        for batch in split_paths(*paths):
+            try:
+                return self.repo.git.check_ignore(*batch).split()
+            except GitCommandError:
+                pass
 
     def find_attr(self, *paths):
         """Return map with path and its attributes."""
         from git.exc import GitCommandError
 
         attrs = defaultdict(dict)
-        try:
-            data = self.repo.git.check_attr("-z", "-a", "--", *paths)
-            for file, name, value in zip_longest(*[iter(data.strip("\0").split("\0"))] * 3):
-                if file:
-                    attrs[file][name] = value
-        except GitCommandError:
-            pass
+
+        for batch in split_paths(*paths):
+            try:
+                data = self.repo.git.check_attr("-z", "-a", "--", *batch)
+                for file, name, value in zip_longest(*[iter(data.strip("\0").split("\0"))] * 3):
+                    if file:
+                        attrs[file][name] = value
+            except GitCommandError:
+                pass
 
         return attrs
 
@@ -213,8 +221,9 @@ class GitCore:
             staged = self.repo.index.diff("HEAD")
 
             for file_path in staged:
-                is_parent = str(file_path.a_path).startswith(path)
-                is_equal = path == file_path.a_path
+                unescaped_path = git_unicode_unescape(file_path.a_path)
+                is_parent = str(unescaped_path).startswith(path)
+                is_equal = path == unescaped_path
 
                 if is_parent or is_equal:
                     raise errors.DirtyRenkuDirectory(self.repo)
@@ -231,7 +240,9 @@ class GitCore:
                 w.set_value("credential", "helper", "cache")
 
     @contextmanager
-    def commit(self, commit_only=None, commit_empty=True, raise_if_empty=False, commit_message=None):
+    def commit(
+        self, commit_only=None, commit_empty=True, raise_if_empty=False, commit_message=None, abbreviate_message=True
+    ):
         """Automatic commit."""
         from git import Actor
 
@@ -240,9 +251,9 @@ class GitCore:
         diff_before = set()
 
         if commit_only == COMMIT_DIFF_STRATEGY:
-            staged = {item.a_path for item in self.repo.index.diff(None)}
+            staged = {git_unicode_unescape(item.a_path) for item in self.repo.index.diff(None)}
 
-            modified = {item.a_path for item in self.repo.index.diff("HEAD")}
+            modified = {git_unicode_unescape(item.a_path) for item in self.repo.index.diff("HEAD")}
 
             if staged or modified:
                 self.repo.git.reset()
@@ -269,10 +280,12 @@ class GitCore:
 
         if commit_only == COMMIT_DIFF_STRATEGY:
             # Get diff generated in command.
-            change_types = {item.a_path: item.change_type for item in self.repo.index.diff(None)}
+            change_types = {git_unicode_unescape(item.a_path): item.change_type for item in self.repo.index.diff(None)}
             staged_after = set(change_types.keys())
 
-            modified_after_change_types = {item.a_path: item.change_type for item in self.repo.index.diff("HEAD")}
+            modified_after_change_types = {
+                git_unicode_unescape(item.a_path): item.change_type for item in self.repo.index.diff("HEAD")
+            }
 
             modified_after = set(modified_after_change_types.keys())
 
@@ -294,7 +307,7 @@ class GitCore:
 
         diffs = []
         try:
-            diffs = [d.a_path for d in self.repo.index.diff("HEAD")]
+            diffs = [git_unicode_unescape(d.a_path) for d in self.repo.index.diff("HEAD")]
             if project_metadata_path in diffs:
                 diffs.remove(project_metadata_path)
         except git.exc.BadName:
@@ -313,6 +326,9 @@ class GitCore:
 
             commit_message = " ".join(argv)
 
+        if abbreviate_message:
+            commit_message = shorten_message(commit_message)
+
         try:
             project = self.project
             if project:
