@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -22,7 +22,19 @@ import os
 from marshmallow import EXCLUDE, post_load, pre_load
 
 from renku.core.models import jsonld
-from renku.core.models.calamus import JsonLDSchema, fields, prov, rdfs, renku, schema, wfprov
+from renku.core.models.calamus import (
+    DateTimeList,
+    JsonLDSchema,
+    StringList,
+    Uri,
+    fields,
+    prov,
+    rdfs,
+    renku,
+    schema,
+    wfprov,
+)
+from renku.core.models.datasets import generate_dataset_tag_id, generate_url_id
 from renku.core.models.git import get_user_info
 from renku.core.models.projects import generate_project_id
 from renku.core.models.provenance.agents import generate_person_id
@@ -33,6 +45,10 @@ class Base:
 
     def __init__(self, **kwargs):
         """Initialize an instance."""
+        self.client = None
+
+        kwargs.setdefault("_id", None)
+
         for k, v in kwargs.items():
             setattr(self, k, v)
 
@@ -40,15 +56,17 @@ class Base:
 class Person(Base):
     """Person migration model."""
 
-    client = None
+    affiliation = None
+    email = None
+    name = None
 
     @staticmethod
     def _fix_person_id(person, client=None):
         """Fixes the id of a Person if it is not set."""
-        if not person._id or "mailto:None" in person._id:
+        if not person._id or "mailto:None" in person._id or person._id.startswith("_:"):
             if not client and person.client:
                 client = person.client
-            person._id = generate_person_id(email=person.email, client=client)
+            person._id = generate_person_id(client=client, email=person.email, full_identity=person.full_identity)
 
         return person
 
@@ -67,10 +85,19 @@ class Person(Base):
         kwargs.setdefault("_id", None)
         super().__init__(**kwargs)
 
+    @property
+    def full_identity(self):
+        """Return name, email, and affiliation."""
+        email = f" <{self.email}>" if self.email else ""
+        affiliation = f" [{self.affiliation}]" if self.affiliation else ""
+        return f"{self.name}{email}{affiliation}"
+
 
 class Project(Base):
     """Project migration model."""
 
+    agent_version = None
+
     @classmethod
     def from_yaml(cls, path, client):
         """Read content from YAML file."""
@@ -80,6 +107,9 @@ class Project(Base):
         if not self.creator:
             self.creator = Person.from_git(client.repo)
 
+        if not self.name:
+            self.name = client.remote.get("name")
+
         if not self._id or "NULL/NULL" in self._id:
             self._id = generate_project_id(client=client, name=self.name, creator=self.creator)
 
@@ -87,6 +117,10 @@ class Project(Base):
 
     def to_yaml(self, path):
         """Write content to a YAML file."""
+        from renku import __version__
+
+        self.agent_version = __version__
+
         data = ProjectSchemaV3().dump(self)
         jsonld.write_yaml(path=path, data=data)
 
@@ -98,6 +132,16 @@ class DatasetFile(Base):
 class DatasetTag(Base):
     """DatasetTag migration model."""
 
+    commit = None
+    name = None
+
+    def __init__(self, **kwargs):
+        """Initialize an instance."""
+        super().__init__(**kwargs)
+
+        if not self._id or self._id.startswith("_:"):
+            self._id = generate_dataset_tag_id(client=self.client, name=self.name, commit=self.commit)
+
 
 class Language(Base):
     """Language migration model."""
@@ -106,6 +150,22 @@ class Language(Base):
 class Url(Base):
     """Url migration model."""
 
+    url = None
+    url_id = None
+    url_str = None
+
+    def __init__(self, **kwargs):
+        """Initialize an instance."""
+        super().__init__(**kwargs)
+
+        if isinstance(self.url, dict):
+            self.url_id = self.url["@id"]
+        elif isinstance(self.url, str):
+            self.url_str = self.url
+
+        if not self._id or self._id.startswith("_:"):
+            self._id = generate_url_id(client=self.client, url_str=self.url_str, url_id=self.url_id)
+
 
 class Dataset(Base):
     """Dataset migration model."""
@@ -115,7 +175,7 @@ class Dataset(Base):
         """Read content from YAML file."""
         data = jsonld.read_yaml(path)
         self = DatasetSchemaV3(client=client, commit=commit).load(data)
-        self.__reference__ = path
+        self._metadata_path = path
         return self
 
     def to_yaml(self, path=None):
@@ -123,7 +183,7 @@ class Dataset(Base):
         from renku.core.management import LocalClient
 
         data = DatasetSchemaV3().dump(self)
-        path = path or self.__reference__ or os.path.join(self.path, LocalClient.METADATA)
+        path = path or self._metadata_path or os.path.join(self.path, LocalClient.METADATA)
         jsonld.write_yaml(path=path, data=data)
 
 
@@ -138,11 +198,11 @@ class PersonSchemaV3(JsonLDSchema):
         unknown = EXCLUDE
 
     _id = fields.Id()
-    name = fields.String(schema.name)
+    name = StringList(schema.name)
     email = fields.String(schema.email, missing=None)
-    label = fields.String(rdfs.label)
-    affiliation = fields.String(schema.affiliation, missing=None)
-    alternate_name = fields.String(schema.alternateName, missing=None)
+    label = StringList(rdfs.label, missing=None)
+    affiliation = StringList(schema.affiliation, missing=None)
+    alternate_name = StringList(schema.alternateName, missing=None)
 
     @post_load
     def make_instance(self, data, **kwargs):
@@ -164,10 +224,10 @@ class ProjectSchemaV3(JsonLDSchema):
         unknown = EXCLUDE
 
     _id = fields.Id(missing=None)
+    agent_version = fields.String(schema.agent, missing="pre-0.11.0")
     name = fields.String(schema.name, missing=None)
-    created = fields.DateTime(schema.dateCreated, missing=None)
-    updated = fields.DateTime(schema.dateUpdated, missing=None)
-    version = fields.String(schema.schemaVersion, missing=1)
+    created = DateTimeList(schema.dateCreated, missing=None)
+    version = StringList(schema.schemaVersion, missing="1")
     creator = fields.Nested(schema.creator, PersonSchemaV3, missing=None)
 
 
@@ -183,7 +243,7 @@ class CommitMixinSchemaV3(JsonLDSchema):
     _id = fields.Id(missing=None)
     _label = fields.String(rdfs.label, missing=None)
     _project = fields.Nested(schema.isPartOf, ProjectSchemaV3, missing=None)
-    path = fields.String(prov.atLocation)
+    path = fields.String(prov.atLocation, missing=None)
 
 
 class EntitySchemaV3(CommitMixinSchemaV3):
@@ -212,7 +272,7 @@ class DatasetFileSchemaV3(EntitySchemaV3):
     external = fields.Boolean(renku.external, missing=False)
 
 
-class LanguageSchemaV5(JsonLDSchema):
+class LanguageSchemaV3(JsonLDSchema):
     """Language schema."""
 
     class Meta:
@@ -226,7 +286,7 @@ class LanguageSchemaV5(JsonLDSchema):
     name = fields.String(schema.name)
 
 
-class DatasetTagSchemaV5(JsonLDSchema):
+class DatasetTagSchemaV3(JsonLDSchema):
     """DatasetTag schema."""
 
     class Meta:
@@ -244,7 +304,7 @@ class DatasetTagSchemaV5(JsonLDSchema):
     name = fields.String(schema.name)
 
 
-class UrlSchemaV5(JsonLDSchema):
+class UrlSchemaV3(JsonLDSchema):
     """Url schema."""
 
     class Meta:
@@ -255,7 +315,7 @@ class UrlSchemaV5(JsonLDSchema):
         unknown = EXCLUDE
 
     _id = fields.Id(missing=None)
-    url = fields.Uri(schema.url, missing=None)
+    url = Uri(schema.url, missing=None)
 
 
 class DatasetSchemaV3(CreatorMixinSchemaV3, EntitySchemaV3):
@@ -268,20 +328,20 @@ class DatasetSchemaV3(CreatorMixinSchemaV3, EntitySchemaV3):
         model = Dataset
         unknown = EXCLUDE
 
-    creators = fields.Nested(schema.creator, PersonSchemaV3, many=True)
+    creators = fields.Nested(schema.creator, PersonSchemaV3, many=True, missing=None)
     date_created = fields.DateTime(schema.dateCreated, missing=None)
     date_published = fields.DateTime(schema.datePublished, missing=None)
     description = fields.String(schema.description, missing=None)
     files = fields.Nested(schema.hasPart, DatasetFileSchemaV3, many=True)
     identifier = fields.String(schema.identifier)
-    in_language = fields.Nested(schema.inLanguage, LanguageSchemaV5, missing=None)
-    keywords = fields.List(schema.keywords, fields.String())
-    license = fields.Uri(schema.license, missing=None, allow_none=True)
+    in_language = fields.Nested(schema.inLanguage, LanguageSchemaV3, missing=None)
+    keywords = fields.List(schema.keywords, fields.String(), missing=None)
+    license = Uri(schema.license, missing=None, allow_none=True)
     name = fields.String(schema.alternateName, missing=None)
-    same_as = fields.Nested(schema.sameAs, UrlSchemaV5, missing=None)
-    tags = fields.Nested(schema.subjectOf, DatasetTagSchemaV5, many=True)
+    same_as = fields.Nested(schema.sameAs, UrlSchemaV3, missing=None)
+    tags = fields.Nested(schema.subjectOf, DatasetTagSchemaV3, many=True, missing=None)
     title = fields.String(schema.name)
-    url = fields.String(schema.url)
+    url = fields.String(schema.url, missing=None)
     version = fields.String(schema.version, missing=None)
 
     @pre_load
@@ -317,4 +377,10 @@ class DatasetSchemaV3(CreatorMixinSchemaV3, EntitySchemaV3):
 def get_client_datasets(client):
     """Return Dataset migration models for a client."""
     paths = client.renku_datasets_path.rglob(client.METADATA)
-    return [Dataset.from_yaml(path=path, client=client) for path in paths]
+    datasets = []
+    for path in paths:
+        dataset = Dataset.from_yaml(path=path, client=client)
+        dataset.path = getattr(dataset, "path", None) or os.path.relpath(path.parent, client.path)
+        datasets.append(dataset)
+
+    return datasets
