@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -16,6 +16,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Client for handling a local repository."""
+import hashlib
+import json
 import os
 import shutil
 import subprocess
@@ -33,25 +35,29 @@ from werkzeug.utils import cached_property, secure_filename
 from renku.core import errors
 from renku.core.compat import Path
 from renku.core.management.config import RENKU_HOME
-from renku.core.models.locals import with_reference
+from renku.core.models.enums import ConfigFilter
 from renku.core.models.projects import Project
+from renku.core.models.provenance.activity import ActivityCollection
+from renku.core.models.provenance.provenance_graph import ProvenanceGraph
 from renku.core.models.refs import LinkReference
+from renku.core.models.workflow.dependency_graph import DependencyGraph
+from renku.core.utils.migrate import MigrationType
 
 from .git import GitCore
 
 DEFAULT_DATA_DIR = "data"
 
 
-def default_path():
+def default_path(path="."):
     """Return default repository path."""
     from git import InvalidGitRepositoryError
 
     from renku.core.commands.git import get_git_home
 
     try:
-        return get_git_home()
+        return get_git_home(path=path)
     except InvalidGitRepositoryError:
-        return "."
+        return path
 
 
 def path_converter(path):
@@ -99,9 +105,20 @@ class RepositoryApiMixin(GitCore):
     WORKFLOW = "workflow"
     """Directory for storing workflow in Renku."""
 
+    DEPENDENCY_GRAPH = "dependency.json"
+    """File for storing dependency graph."""
+
+    PROVENANCE_GRAPH = "provenance.json"
+    """File for storing ProvenanceGraph."""
+
     ACTIVITY_INDEX = "activity_index.yaml"
     """Caches activities that generated a path."""
 
+    DOCKERFILE = "Dockerfile"
+    """Name of the Dockerfile in the repo."""
+
+    TEMPLATE_CHECKSUMS = "template_checksums.json"
+
     RENKU_PROTECTED_PATHS = [
         "\\.renku/.*",
         "Dockerfile",
@@ -113,11 +130,15 @@ class RepositoryApiMixin(GitCore):
         "requirements\\.txt",
     ]
 
-    _commit_activity_cache = {}
+    _commit_activity_cache = attr.ib(factory=dict)
 
-    _activity_index = None
+    _activity_index = attr.ib(default=None)
 
-    _remote_cache = {}
+    _remote_cache = attr.ib(factory=dict)
+
+    _dependency_graph = None
+
+    _migration_type = attr.ib(default=MigrationType.ALL)
 
     def __attrs_post_init__(self):
         """Initialize computed attributes."""
@@ -129,7 +150,7 @@ class RepositoryApiMixin(GitCore):
         path.relative_to(path)
         self.renku_path = path
 
-        data_dir = self.get_value("renku", self.DATA_DIR_CONFIG_KEY, local_only=True)
+        data_dir = self.get_value("renku", self.DATA_DIR_CONFIG_KEY, config_filter=ConfigFilter.LOCAL_ONLY)
         self.data_dir = data_dir or self.data_dir
 
         self._subclients = {}
@@ -148,13 +169,30 @@ class RepositoryApiMixin(GitCore):
     @property
     def latest_agent(self):
         """Returns latest agent version used in the repository."""
-        return self.project.agent_version
+        try:
+            return self.project.agent_version
+        except ValueError as e:
+            if "Project name not set" in str(e):
+                return None
+            raise
 
     @property
     def lock(self):
         """Create a Renku config lock."""
         return filelock.FileLock(str(self.renku_path.with_suffix(self.LOCK_SUFFIX)), timeout=0,)
 
+    @property
+    def migration_type(self):
+        """Type of migration that is being executed on this client."""
+        return self._migration_type
+
+    @migration_type.setter
+    def migration_type(self, value):
+        """Set type of migration."""
+        if not isinstance(value, MigrationType):
+            raise ValueError(f"Invalid value for MigrationType: {type(value)}")
+        self._migration_type = value
+
     @property
     def renku_metadata_path(self):
         """Return a ``Path`` instance of Renku metadata file."""
@@ -170,12 +208,42 @@ class RepositoryApiMixin(GitCore):
         """Path to the activity filepath cache."""
         return self.renku_path / self.ACTIVITY_INDEX
 
+    @property
+    def docker_path(self):
+        """Path to the Dockerfile."""
+        return self.path / self.DOCKERFILE
+
+    @property
+    def template_checksums(self):
+        """Return a ``Path`` instance to the template checksums file."""
+        return self.renku_path / self.TEMPLATE_CHECKSUMS
+
+    @property
+    def provenance_graph_path(self):
+        """Path to store activity files."""
+        return self.renku_path / self.PROVENANCE_GRAPH
+
+    @property
+    def dependency_graph_path(self):
+        """Path to the dependency graph file."""
+        return self.renku_path / self.DEPENDENCY_GRAPH
+
     @cached_property
     def cwl_prefix(self):
         """Return a CWL prefix."""
         self.workflow_path.mkdir(parents=True, exist_ok=True)  # for Python 3.5
         return str(self.workflow_path.resolve().relative_to(self.path))
 
+    @property
+    def dependency_graph(self):
+        """Return dependency graph if available."""
+        if not self.has_graph_files():
+            return
+        if not self._dependency_graph:
+            self._dependency_graph = DependencyGraph.from_json(self.dependency_graph_path)
+
+        return self._dependency_graph
+
     @property
     def project(self):
         """Return the Project instance."""
@@ -389,12 +457,11 @@ class RepositoryApiMixin(GitCore):
             metadata = Project.from_yaml(metadata_path, client=self)
         else:
             metadata = Project(name=name, client=self)
-            metadata.__reference__ = metadata_path
 
         yield metadata
 
         if not read_only:
-            metadata.to_yaml()
+            metadata.to_yaml(path=metadata_path)
 
     @contextmanager
     def with_workflow_storage(self):
@@ -413,10 +480,58 @@ class RepositoryApiMixin(GitCore):
 
             path = workflow_path / step_name
 
-            with with_reference(path):
-                run = step.run.generate_process_run(client=self, commit=self.repo.head.commit, path=path,)
-                run.to_yaml()
-                self.add_to_activity_index(run)
+            run = step.run.generate_process_run(client=self, commit=self.repo.head.commit, path=path,)
+            run.to_yaml(path=path)
+            self.add_to_activity_index(run)
+
+    def process_and_store_run(self, command_line_tool, name, client):
+        """Create Plan and Activity from CommandLineTool and store them."""
+        filename = "{0}_{1}.yaml".format(uuid.uuid4().hex, secure_filename("_".join(command_line_tool.baseCommand)))
+
+        # Store Run and ProcessRun as before
+        self.workflow_path.mkdir(exist_ok=True)
+        path = self.workflow_path / filename
+
+        process_run = command_line_tool.generate_process_run(client=self, commit=self.repo.head.commit, path=path)
+        process_run.to_yaml(path=path)
+        self.add_to_activity_index(process_run)
+
+        self.update_graphs(process_run)
+
+    def update_graphs(self, activity_run):
+        """Update Dependency and Provenance graphs from a ProcessRun/WorkflowRun."""
+        if not self.has_graph_files():
+            return
+
+        dependency_graph = DependencyGraph.from_json(self.dependency_graph_path)
+        provenance_graph = ProvenanceGraph.from_json(self.provenance_graph_path)
+
+        activity_collection = ActivityCollection.from_activity_run(activity_run, dependency_graph, self)
+
+        provenance_graph.add(activity_collection)
+
+        dependency_graph.to_json()
+        provenance_graph.to_json()
+
+    def has_graph_files(self):
+        """Return true if dependency or provenance graph exists."""
+        return self.dependency_graph_path.exists() or self.provenance_graph_path.exists()
+
+    def initialize_graph(self):
+        """Create empty graph files."""
+        self.dependency_graph_path.write_text("[]")
+        self.provenance_graph_path.write_text("[]")
+
+    def remove_graph_files(self):
+        """Remove all graph files."""
+        try:
+            self.dependency_graph_path.unlink()
+        except FileNotFoundError:
+            pass
+        try:
+            self.provenance_graph_path.unlink()
+        except FileNotFoundError:
+            pass
 
     def init_repository(self, force=False, user=None):
         """Initialize an empty Renku repository."""
@@ -524,8 +639,10 @@ class RepositoryApiMixin(GitCore):
 
     def import_from_template(self, template_path, metadata, force=False):
         """Render template files from a template directory."""
+        checksums = {}
         for file in template_path.glob("**/*"):
-            destination = self.path / file.relative_to(template_path)
+            rel_path = file.relative_to(template_path)
+            destination = self.path / rel_path
             try:
                 # TODO: notify about the expected variables - code stub:
                 # with file.open() as fr:
@@ -538,9 +655,24 @@ class RepositoryApiMixin(GitCore):
                 # parse file and process it
                 template = Template(file.read_text())
                 rendered_content = template.render(metadata)
+                # NOTE: the path could contain template variables, we need to template it
                 destination = Path(Template(str(destination)).render(metadata))
                 destination.write_text(rendered_content)
+                checksums[str(rel_path)] = self._content_hash(destination)
             except IsADirectoryError:
                 destination.mkdir(parents=True, exist_ok=True)
             except TypeError:
                 shutil.copy(file, destination)
+
+        self.template_checksums.parent.mkdir(parents=True, exist_ok=True)
+
+        with open(self.template_checksums, "w") as checksum_file:
+            json.dump(checksums, checksum_file)
+
+    def _content_hash(self, path):
+        """Calculate the sha256 hash of a file."""
+        sha256_hash = hashlib.sha256()
+        with open(str(path), "rb") as f:
+            for byte_block in iter(lambda: f.read(4096), b""):
+                sha256_hash.update(byte_block)
+        return sha256_hash.hexdigest()
