@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -19,7 +19,6 @@
 
 import os
 import urllib
-import uuid
 import weakref
 from collections import OrderedDict
 from pathlib import Path, posixpath
@@ -29,7 +28,7 @@ from git import NULL_TREE
 from marshmallow import EXCLUDE
 
 from renku.core.models import jsonld
-from renku.core.models.calamus import Nested, fields, oa, prov, rdfs, wfprov
+from renku.core.models.calamus import Nested, fields, oa, prov, rdfs, renku, wfprov
 from renku.core.models.cwl.annotation import AnnotationSchema
 from renku.core.models.entities import (
     Collection,
@@ -39,10 +38,11 @@ from renku.core.models.entities import (
     Entity,
     EntitySchema,
 )
-from renku.core.models.locals import ReferenceMixin
 from renku.core.models.refs import LinkReference
 from renku.core.models.workflow.run import Run
+from renku.core.utils.scm import git_unicode_unescape
 
+from ..workflow.parameters import RunParameter, RunParameterSchema
 from .agents import Person, PersonSchema, SoftwareAgentSchema, renku_agent
 from .qualified import Association, AssociationSchema, Generation, GenerationSchema, Usage, UsageSchema
 
@@ -82,7 +82,7 @@ def _set_entity_client_commit(entity, client, commit):
     if not entity.commit:
         revision = "UNCOMMITTED"
         if entity._label:
-            revision = entity._label.rsplit("@")[1]
+            revision = entity._label.rsplit("@", maxsplit=1)[-1]
         if revision == "UNCOMMITTED":
             commit = commit
         elif client:
@@ -90,8 +90,8 @@ def _set_entity_client_commit(entity, client, commit):
         entity.commit = commit
 
 
-@attr.s(cmp=False,)
-class Activity(CommitMixin, ReferenceMixin):
+@attr.s(eq=False, order=False)
+class Activity(CommitMixin):
     """Represent an activity in the repository."""
 
     _id = attr.ib(default=None, kw_only=True)
@@ -113,6 +113,8 @@ class Activity(CommitMixin, ReferenceMixin):
 
     agents = attr.ib(kw_only=True)
 
+    _metadata_path = attr.ib(default=None, init=False)
+
     def default_generated(self):
         """Create default ``generated``."""
         generated = []
@@ -139,15 +141,27 @@ class Activity(CommitMixin, ReferenceMixin):
             # in this backwards diff
             if file_.change_type == "A":
                 continue
-            path_ = Path(file_.a_path)
 
-            is_dataset = self.client.DATASETS in str(path_)
+            path_ = Path(git_unicode_unescape(file_.a_path))
+
+            is_dataset = any(
+                [
+                    path_.resolve() == (self.client.path / f.path).resolve()
+                    for d in self.client.datasets.values()
+                    for f in d.files
+                ]
+            )
             not_refs = LinkReference.REFS not in str(path_)
             does_not_exists = not path_.exists()
 
             if all([is_dataset, not_refs, does_not_exists]):
-                uid = uuid.UUID(path_.parent.name)
-                path_ = Path(self.client.renku_home) / self.client.DATASETS / str(uid) / self.client.METADATA
+                dataset = next(
+                    d
+                    for d in self.client.datasets.values()
+                    for f in d.files
+                    if path_.resolve() == (self.client.path / f.path).resolve()
+                )
+                path_ = self.client.path / dataset.path / self.client.METADATA
 
             index.add(str(path_))
 
@@ -218,7 +232,7 @@ class Activity(CommitMixin, ReferenceMixin):
             # in this backwards diff
             if file_.change_type != "A":
                 continue
-            path_ = Path(file_.a_path)
+            path_ = Path(git_unicode_unescape(file_.a_path))
 
             index.add(str(path_))
 
@@ -234,15 +248,26 @@ class Activity(CommitMixin, ReferenceMixin):
             # in this backwards diff
             if file_.change_type == "A":
                 continue
-            path_ = Path(file_.a_path)
+            path_ = Path(git_unicode_unescape(file_.a_path))
 
-            is_dataset = self.client.DATASETS in str(path_)
+            is_dataset = any(
+                [
+                    path_.resolve() == (self.client.path / f.path).resolve()
+                    for d in self.client.datasets.values()
+                    for f in d.files
+                ]
+            )
             not_refs = LinkReference.REFS not in str(path_)
             does_not_exists = not (path_.exists() or (path_.is_symlink() and os.path.lexists(path_)))
 
             if all([is_dataset, not_refs, does_not_exists]):
-                uid = uuid.UUID(path_.parent.name)
-                path_ = Path(self.client.renku_home) / self.client.DATASETS / str(uid) / self.client.METADATA
+                dataset = next(
+                    d
+                    for d in self.client.datasets
+                    for f in d.files
+                    if path_.resolve() == (self.client.path / f.path).resolve()
+                )
+                path_ = self.client.path / dataset.path / self.client.METADATA
 
             index.add(str(path_))
 
@@ -346,14 +371,15 @@ class Activity(CommitMixin, ReferenceMixin):
         data = jsonld.read_yaml(path)
 
         self = cls.from_jsonld(data=data, client=client, commit=commit)
-        self.__reference__ = path
+        self._metadata_path = path
 
         return self
 
-    def to_yaml(self):
+    def to_yaml(self, path=None):
         """Write an instance to the referenced YAML file."""
+        self._metadata_path = path or self._metadata_path
         data = ActivitySchema(flattened=True).dump(self)
-        jsonld.write_yaml(path=self.__reference__, data=data)
+        jsonld.write_yaml(path=self._metadata_path, data=data)
 
     @classmethod
     def from_jsonld(cls, data, client=None, commit=None):
@@ -377,7 +403,7 @@ class Activity(CommitMixin, ReferenceMixin):
         return ActivitySchema(flattened=True).dump(self)
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
 class ProcessRun(Activity):
     """A process run is a particular execution of a Process description."""
 
@@ -391,6 +417,8 @@ class ProcessRun(Activity):
 
     qualified_usage = attr.ib(kw_only=True, default=None)
 
+    run_parameter = attr.ib(kw_only=True, default=None)
+
     def __attrs_post_init__(self):
         """Calculate properties."""
         super().__attrs_post_init__()
@@ -431,7 +459,7 @@ class ProcessRun(Activity):
                     if not usage.client:
                         usage.entity.set_client(self.client)
                     if not usage.commit:
-                        revision = usage._label.rsplit("@")[1]
+                        revision = usage._label.rsplit("@", maxsplit=1)[-1]
                         usage.entity.commit = self.client.repo.commit(revision)
 
                     usages.append(usage)
@@ -496,8 +524,20 @@ class ProcessRun(Activity):
         agent = SoftwareAgent.from_commit(commit)
         association = Association(agent=agent, id=id_ + "/association", plan=run)
 
+        run_parameter = []
+
+        for parameter in run.run_parameters:
+            parameter_id = f"{id_}/{parameter.name}"
+            run_parameter.append(RunParameter(name=parameter.name, value=parameter.value, id=parameter_id))
+
         process_run = cls(
-            id=id_, qualified_usage=usages, association=association, client=client, commit=commit, path=path
+            id=id_,
+            qualified_usage=usages,
+            association=association,
+            client=client,
+            commit=commit,
+            path=path,
+            run_parameter=run_parameter,
         )
 
         generated = []
@@ -527,10 +567,11 @@ class ProcessRun(Activity):
         # Activity itself
         yield self.association.plan
 
-    def to_yaml(self):
+    def to_yaml(self, path=None):
         """Write an instance to the referenced YAML file."""
+        self._metadata_path = path or self._metadata_path
         data = ProcessRunSchema(flattened=True).dump(self)
-        jsonld.write_yaml(path=self.__reference__, data=data)
+        jsonld.write_yaml(path=self._metadata_path, data=data)
 
     @classmethod
     def from_jsonld(cls, data, client=None, commit=None):
@@ -547,7 +588,7 @@ class ProcessRun(Activity):
         return ProcessRunSchema(flattened=True).dump(self)
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
 class WorkflowRun(ProcessRun):
     """A workflow run typically contains several subprocesses."""
 
@@ -561,7 +602,7 @@ class WorkflowRun(ProcessRun):
         return {i: p for i, p in enumerate(self._processes)}
 
     @classmethod
-    def from_run(cls, run, client, path, commit=None, update_commits=False):
+    def from_run(cls, run, client, path, commit=None, subprocess_index=None, update_commits=False):
         """Convert a ``Run`` to a ``WorkflowRun``."""
         from .agents import SoftwareAgent
 
@@ -665,10 +706,11 @@ class WorkflowRun(ProcessRun):
 
         super().__attrs_post_init__()
 
-    def to_yaml(self):
+    def to_yaml(self, path=None):
         """Write an instance to the referenced YAML file."""
+        self._metadata_path = path or self._metadata_path
         data = WorkflowRunSchema(flattened=True).dump(self)
-        jsonld.write_yaml(path=self.__reference__, data=data)
+        jsonld.write_yaml(path=self._metadata_path, data=data)
 
     @classmethod
     def from_jsonld(cls, data, client=None, commit=None):
@@ -698,7 +740,7 @@ class ActivitySchema(CommitMixinSchema):
     _message = fields.String(rdfs.comment, init_name="message", missing=None)
     _was_informed_by = fields.List(prov.wasInformedBy, fields.IRI(), init_name="was_informed_by")
     generated = Nested(prov.activity, GenerationSchema, reverse=True, many=True, missing=None)
-    invalidated = Nested(prov.wasInvalidatedBy, EntitySchema, reverse=True, many=True, missing=None)
+    invalidated = Nested(prov.wasInvalidatedBy, [EntitySchema, CollectionSchema], reverse=True, many=True, missing=None)
     influenced = Nested(prov.influenced, CollectionSchema, many=True)
     started_at_time = fields.DateTime(prov.startedAtTime, add_value_types=True)
     ended_at_time = fields.DateTime(prov.endedAtTime, add_value_types=True)
@@ -718,6 +760,7 @@ class ProcessRunSchema(ActivitySchema):
     association = Nested(prov.qualifiedAssociation, AssociationSchema)
     annotations = Nested(oa.hasTarget, AnnotationSchema, reverse=True, many=True)
     qualified_usage = Nested(prov.qualifiedUsage, UsageSchema, many=True)
+    run_parameter = Nested(renku.hasRunParameter, RunParameterSchema, many=True)
 
 
 class WorkflowRunSchema(ProcessRunSchema):
