@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -123,7 +123,10 @@ class CWLConverter(object):
                     continue
 
                 for previous_process in previous_output_dirs[key]:
-                    previous_process.outputs.append(CommandOutput(produces=entity, create_folder=False))
+                    new_output = CommandOutput(
+                        id=CommandOutput.generate_id(subprocess._id), produces=entity, create_folder=False
+                    )
+                    previous_process.outputs.append(new_output)
 
             for output in subprocess.outputs:
                 entity = output.produces
@@ -279,6 +282,36 @@ class CWLConverter(object):
         for argument in step.arguments:
             tool_object.inputs.append(CWLConverter._convert_argument(argument))
 
+        workdir_req.listing.append(
+            cwlgen.InitialWorkDirRequirement.Dirent(
+                entry="$(inputs.input_renku_metadata)", entryname=".renku", writable=False
+            )
+        )
+        tool_object.inputs.append(
+            cwlgen.CommandInputParameter(
+                "input_renku_metadata",
+                param_type="Directory",
+                input_binding=None,
+                default={"path": os.path.abspath(os.path.join(basedir, ".renku")), "class": "Directory"},
+            )
+        )
+
+        # TODO: ".git" is not required once https://github.com/SwissDataScienceCenter/renku-python/issues/1043 is done
+        # because we won't need the git history to correctly load metadata. The following two statements can be removed.
+        workdir_req.listing.append(
+            cwlgen.InitialWorkDirRequirement.Dirent(
+                entry="$(inputs.input_git_directory)", entryname=".git", writable=False
+            )
+        )
+        tool_object.inputs.append(
+            cwlgen.CommandInputParameter(
+                "input_git_directory",
+                param_type="Directory",
+                input_binding=None,
+                default={"path": os.path.abspath(os.path.join(basedir, ".git")), "class": "Directory"},
+            )
+        )
+
         if workdir_req.listing:
             tool_object.requirements.append(workdir_req)
         if jsrequirement:
