@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2018-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -25,11 +25,11 @@ import uuid
 import attr
 from marshmallow import EXCLUDE
 
-from renku.core.models.calamus import JsonLDSchema, Nested, fields, rdfs, renku
+from renku.core.models.calamus import JsonLDSchema, Nested, fields, rdfs, renku, schema
 from renku.core.models.entities import CollectionSchema, EntitySchema
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
 class MappedIOStream(object):
     """Represents an IO stream (stdin, stdout, stderr)."""
 
@@ -79,7 +79,7 @@ class MappedIOStream(object):
         return MappedIOStreamSchema().dump(self)
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
 class CommandParameter(object):
     """Represents a parameter for an execution template."""
 
@@ -98,7 +98,7 @@ class CommandParameter(object):
         return "/".join(self._id.split("/")[-2:])
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
 class CommandArgument(CommandParameter):
     """An argument to a command that is neither input nor output."""
 
@@ -146,7 +146,7 @@ class CommandArgument(CommandParameter):
         return CommandArgumentSchema().dump(self)
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
 class CommandInput(CommandParameter):
     """An input to a command."""
 
@@ -203,7 +203,56 @@ class CommandInput(CommandParameter):
         return CommandInputSchema().dump(self)
 
 
-@attr.s(cmp=False,)
+@attr.s(eq=False, order=False)
+class CommandInputTemplate(CommandParameter):
+    """Template for inputs of a Plan."""
+
+    consumes = attr.ib(kw_only=True,)
+    mapped_to = attr.ib(default=None, kw_only=True)
+
+    @staticmethod
+    def generate_id(plan_id, position=None, id_=None):
+        """Generate an id for an argument."""
+        if not id_:
+            id_ = str(position) if position else uuid.uuid4().hex
+        return f"{plan_id}/inputs/{id_}"
+
+    def default_label(self):
+        """Set default label."""
+        return 'Command Input Template "{}"'.format(self.consumes)
+
+    def to_argv(self):
+        """String representation (sames as cmd argument)."""
+        raise RuntimeError("Cannot use CommandInputTemplate in a command.")
+
+    def to_stream_repr(self):
+        """Input stream representation."""
+        if not self.mapped_to:
+            return ""
+
+        return " < {}".format(self.consumes)
+
+    def __attrs_post_init__(self):
+        """Post-init hook."""
+        if not self._label:
+            self._label = self.default_label()
+
+    @classmethod
+    def from_jsonld(cls, data):
+        """Create an instance from JSON-LD data."""
+        if isinstance(data, cls):
+            return data
+        if not isinstance(data, dict):
+            raise ValueError(data)
+
+        return CommandInputTemplateSchema().load(data)
+
+    def as_jsonld(self):
+        """Create JSON-LD."""
+        return CommandInputTemplateSchema().dump(self)
+
+
+@attr.s(eq=False, order=False)
 class CommandOutput(CommandParameter):
     """An output of a command."""
 
@@ -265,6 +314,106 @@ class CommandOutput(CommandParameter):
         return CommandOutputSchema().dump(self)
 
 
+@attr.s(eq=False, order=False)
+class RunParameter:
+    """A run parameter that is set inside the script."""
+
+    _id = attr.ib(default=None, kw_only=True)
+
+    _label = attr.ib(default=None, kw_only=True)
+
+    name = attr.ib(default=None, type=str, kw_only=True)
+
+    value = attr.ib(default=None, type=str, kw_only=True)
+
+    type = attr.ib(default=None, type=str, kw_only=True)
+
+    @staticmethod
+    def generate_id(run_id, name):
+        """Generate an id."""
+        name = urllib.parse.quote(name, safe="")
+        return "{}/parameters/{}".format(run_id, name)
+
+    def default_label(self):
+        """Set default label."""
+        return 'Run Parameter "{}"'.format(self.name)
+
+    def __attrs_post_init__(self):
+        """Post-init hook."""
+        if not self._label:
+            self._label = self.default_label()
+
+        if not self.type:
+            self.type = type(self.value).__name__
+
+    @classmethod
+    def from_jsonld(cls, data):
+        """Create an instance from JSON-LD data."""
+        if isinstance(data, cls):
+            return data
+        if not isinstance(data, dict):
+            raise ValueError(data)
+
+        return RunParameterSchema().load(data)
+
+    def as_jsonld(self):
+        """Create JSON-LD."""
+        return RunParameterSchema().dump(self)
+
+
+@attr.s(eq=False, order=False)
+class CommandOutputTemplate(CommandParameter):
+    """Template for outputs of a Plan."""
+
+    create_folder = attr.ib(default=False, kw_only=True, type=bool)
+    mapped_to = attr.ib(default=None, kw_only=True)
+    produces = attr.ib(default=None, kw_only=True, type=str)
+
+    @staticmethod
+    def generate_id(plan_id, position=None, id_=None):
+        """Generate an id for an argument."""
+        if not id_:
+            id_ = str(position) if position else uuid.uuid4().hex
+        return f"{plan_id}/outputs/{id_}"
+
+    def default_label(self):
+        """Set default label."""
+        return 'Command Output Template "{}"'.format(self.produces)
+
+    def to_argv(self):
+        """String representation (sames as cmd argument)."""
+        raise RuntimeError("Cannot use CommandOutputTemplate in a command.")
+
+    def to_stream_repr(self):
+        """Input stream representation."""
+        if not self.mapped_to:
+            return ""
+
+        if self.mapped_to.stream_type == "stdout":
+            return " > {}".format(self.produces)
+
+        return " 2> {}".format(self.produces)
+
+    def __attrs_post_init__(self):
+        """Post-init hook."""
+        if not self._label:
+            self._label = self.default_label()
+
+    @classmethod
+    def from_jsonld(cls, data):
+        """Create an instance from JSON-LD data."""
+        if isinstance(data, cls):
+            return data
+        if not isinstance(data, dict):
+            raise ValueError(data)
+
+        return CommandOutputTemplateSchema().load(data)
+
+    def as_jsonld(self):
+        """Create JSON-LD."""
+        return CommandOutputTemplateSchema().dump(self)
+
+
 class MappedIOStreamSchema(JsonLDSchema):
     """MappedIOStream schema."""
 
@@ -323,6 +472,20 @@ class CommandInputSchema(CommandParameterSchema):
     mapped_to = Nested(renku.mappedTo, MappedIOStreamSchema, missing=None)
 
 
+class CommandInputTemplateSchema(CommandParameterSchema):
+    """CommandInputTemplateSchema schema."""
+
+    class Meta:
+        """Meta class."""
+
+        rdf_type = [renku.CommandInputTemplate]
+        model = CommandInputTemplate
+        unknown = EXCLUDE
+
+    consumes = fields.String(renku.consume)
+    mapped_to = Nested(renku.mappedTo, MappedIOStreamSchema, missing=None)
+
+
 class CommandOutputSchema(CommandParameterSchema):
     """CommandArgument schema."""
 
@@ -336,3 +499,35 @@ class CommandOutputSchema(CommandParameterSchema):
     create_folder = fields.Boolean(renku.createFolder)
     produces = Nested(renku.produces, [EntitySchema, CollectionSchema])
     mapped_to = Nested(renku.mappedTo, MappedIOStreamSchema, missing=None)
+
+
+class CommandOutputTemplateSchema(CommandParameterSchema):
+    """CommandOutputTemplateSchema schema."""
+
+    class Meta:
+        """Meta class."""
+
+        rdf_type = [renku.CommandOutputTemplate]
+        model = CommandOutputTemplate
+        unknown = EXCLUDE
+
+    create_folder = fields.Boolean(renku.createFolder)
+    mapped_to = Nested(renku.mappedTo, MappedIOStreamSchema, missing=None)
+    produces = fields.String(renku.produces)
+
+
+class RunParameterSchema(JsonLDSchema):
+    """CommandParameter schema."""
+
+    class Meta:
+        """Meta class."""
+
+        rdf_type = [renku.RunParameter]
+        model = RunParameter
+        unknown = EXCLUDE
+
+    _id = fields.Id(init_name="id")
+    _label = fields.String(rdfs.label, init_name="label")
+    name = fields.String(schema.name)
+    value = fields.String(renku.value)
+    type = fields.String(renku.type)
