@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2018-2020- Swiss Data Science Center (SDSC)
+# Copyright 2018-2021- Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -15,20 +15,19 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
-"""Implement various context mananagers."""
+"""Implement various context managers."""
 
 import contextlib
 import os
 import sys
 from pathlib import Path
 
+import click
+
 
 @contextlib.contextmanager
 def chdir(path):
     """Change the current working directory."""
-    if isinstance(path, Path):
-        path = str(path)
-
     cwd = os.getcwd()
     os.chdir(path)
     try:
@@ -87,3 +86,30 @@ class Isolation(contextlib.ExitStack):
         for key, value in kwargs.items():
             if value is not None:
                 self.enter_context(self.CONTEXTS[key](value))
+
+
+@contextlib.contextmanager
+def measure(message="TOTAL"):
+    """Measure execution time of enclosing code block."""
+    import time
+
+    start = time.time()
+    try:
+        yield
+    finally:
+        end = time.time()
+        total_seconds = float("%.2f" % (end - start))
+        print(f"{message}: {total_seconds} seconds")
+
+
+def click_context(path, command):
+    """Provide a click context with repo path injected."""
+
+    from renku.core.management import LocalClient
+    from renku.core.management.config import RENKU_HOME
+    from renku.core.management.repository import default_path
+
+    return click.Context(
+        click.Command(command),
+        obj=LocalClient(path=default_path(path), renku_home=RENKU_HOME, external_storage_requested=True),
+    ).scope()
