@@ -17,11 +17,22 @@
 # limitations under the License.
 """Helper utils for migrations."""
 
+from enum import IntFlag
+
 import pyld
 
 from renku.core.models.jsonld import read_yaml
 
 
+class MigrationType(IntFlag):
+    """Type of migration that is being executed."""
+
+    DATASETS = 1
+    WORKFLOWS = 2
+    STRUCTURAL = 4
+    ALL = DATASETS | WORKFLOWS | STRUCTURAL
+
+
 def migrate_types(data):
     """Fix data types."""
     type_mapping = {
@@ -31,14 +42,27 @@ def migrate_types(data):
         "schema:DigitalDocument": ["prov:Entity", "schema:DigitalDocument", "wfprov:Artifact"],
     }
 
-    def replace_types(data):
-        for key, value in data.items():
+    def remove_type(data_):
+        data_.pop("@type", None)
+        for key, value in data_.items():
+            if isinstance(value, dict):
+                remove_type(value)
+            elif isinstance(value, (list, tuple, set)):
+                for v in value:
+                    if isinstance(v, dict):
+                        remove_type(v)
+
+    def replace_types(data_):
+        for key, value in data_.items():
+            if key == "@context" and isinstance(value, dict):
+                remove_type(value)
+
             if key == "@type":
                 if not isinstance(value, str):
                     value = str(sorted(value))
                 new_type = type_mapping.get(value)
                 if new_type:
-                    data[key] = new_type
+                    data_[key] = new_type
             elif isinstance(value, dict):
                 replace_types(value)
             elif isinstance(value, (list, tuple, set)):
@@ -57,23 +81,21 @@ def get_pre_0_3_4_datasets_metadata(client):
 
     project_is_pre_0_3 = int(read_project_version(client)) < 2
     if project_is_pre_0_3:
-        return (client.path / DATA_DIR).rglob(client.METADATA)
+        return (client.path / DATA_DIR).glob(f"*/{client.METADATA}")
     return []
 
 
 def read_project_version(client):
     """Read project version from metadata file."""
-    metadata = read_jsonld_yaml(client.renku_metadata_path)
-    return _get_jsonld_property(metadata, "http://schema.org/schemaVersion", "1")
+    yaml_data = read_yaml(client.renku_metadata_path)
+    return read_project_version_from_yaml(yaml_data)
 
 
-def read_jsonld_yaml(path):
-    """Read YAML file and return normalized expanded JSON-LD."""
-    data = read_yaml(path)
-    jsonld = pyld.jsonld.expand(data)[0]
-    v = normalize(jsonld)
-
-    return v
+def read_project_version_from_yaml(yaml_data):
+    """Read project version from YAML data."""
+    jsonld = pyld.jsonld.expand(yaml_data)[0]
+    jsonld = normalize(jsonld)
+    return _get_jsonld_property(jsonld, "http://schema.org/schemaVersion", "1")
 
 
 def _get_jsonld_property(jsonld, property, default=None):
