@@ -42,16 +42,10 @@ class DatasetsListCtrl(ServiceCtrl, ReadOperationMixin):
 
     def renku_op(self):
         """Renku operation for the controller."""
-        return list_datasets()
+        result = list_datasets().build().execute()
+        return result.output
 
     def to_response(self):
         """Execute controller flow and serialize to service response."""
-        self.ctx["datasets"] = []
-
-        if "project_id" in self.ctx:
-            self.ctx["datasets"] = self.local()
-
-        elif "git_url" in self.ctx:
-            self.ctx["datasets"] = self.remote()
-
+        self.ctx["datasets"] = self.execute_op()
         return result_response(DatasetsListCtrl.RESPONSE_SERIALIZER, self.ctx)
