@@ -27,6 +27,7 @@ from werkzeug.utils import secure_filename
 from renku.core.errors import ConfigurationError
 from renku.core.models.git import GitURL
 from renku.service.config import PROJECT_CLONE_DEPTH_DEFAULT
+from renku.service.serializers.common import RenkuSyncSchema
 from renku.service.serializers.rpc import JsonRPCResponse
 
 
@@ -102,6 +103,7 @@ class ProjectCloneRequest(Schema):
 
     git_url = fields.String(required=True)
     depth = fields.Integer(missing=PROJECT_CLONE_DEPTH_DEFAULT)
+    ref = fields.String(missing="master")
 
 
 class ProjectCloneContext(ProjectCloneRequest):
@@ -112,20 +114,21 @@ class ProjectCloneContext(ProjectCloneRequest):
     # measured in ms
     timestamp = fields.Integer(missing=time.time() * 1e3)
 
-    name = fields.String(required=True)
-    fullname = fields.String(required=True)
-    email = fields.String(required=True)
-    owner = fields.String(required=True)
-    token = fields.String(required=True)
-    ref = fields.String(missing="master")
+    name = fields.String()
+    fullname = fields.String()
+    email = fields.String()
+    owner = fields.String()
+    token = fields.String()
 
     @validates("git_url")
     def validate_git_url(self, value):
         """Validates git url."""
         try:
             GitURL.parse(value)
+        except UnicodeError as e:
+            raise ValidationError("`git_url` contains unsupported characters") from e
         except ConfigurationError as e:
-            raise ValidationError(str(e))
+            raise ValidationError("Invalid `git_url`") from e
 
         return value
 
@@ -142,9 +145,19 @@ class ProjectCloneContext(ProjectCloneRequest):
     @pre_load()
     def set_owner_name(self, data, **kwargs):
         """Set owner and name fields."""
-        git_url = GitURL.parse(data["git_url"])
+        try:
+            git_url = GitURL.parse(data["git_url"])
+        except UnicodeError as e:
+            raise ValidationError("`git_url` contains unsupported characters") from e
+        except ConfigurationError as e:
+            raise ValidationError("Invalid `git_url`") from e
 
+        if git_url.owner is None:
+            raise ValidationError("Invalid `git_url`")
         data["owner"] = git_url.owner
+
+        if git_url.name is None:
+            raise ValidationError("Invalid `git_url`")
         data["name"] = git_url.name
 
         return data
@@ -155,6 +168,7 @@ class ProjectCloneResponse(Schema):
 
     project_id = fields.String(required=True)
     git_url = fields.String(required=True)
+    initialized = fields.Boolean(default=False)
 
 
 class ProjectCloneResponseRPC(JsonRPCResponse):
@@ -179,7 +193,12 @@ class ProjectMigrateRequest(Schema):
     """Request schema for project migrate."""
 
     project_id = fields.String(required=True)
+    force_template_update = fields.Boolean(default=False)
+    skip_template_update = fields.Boolean(default=False)
+    skip_docker_update = fields.Boolean(default=False)
+    skip_migrations = fields.Boolean(default=False)
     is_delayed = fields.Boolean(default=False)
+    client_extras = fields.String()
     commit_message = fields.String()
 
     @pre_load()
@@ -191,10 +210,12 @@ class ProjectMigrateRequest(Schema):
         return data
 
 
-class ProjectMigrateResponse(Schema):
+class ProjectMigrateResponse(RenkuSyncSchema):
     """Response schema for project migrate."""
 
     was_migrated = fields.Boolean()
+    template_migrated = fields.Boolean()
+    docker_migrated = fields.Boolean()
     messages = fields.List(fields.String)
 
 
@@ -217,10 +238,27 @@ class ProjectMigrateAsyncResponseRPC(JsonRPCResponse):
     result = fields.Nested(ProjectMigrateJobResponse)
 
 
+class ProjectMigrationCheckRequest(Schema):
+    """Request schema for project migration check."""
+
+    project_id = fields.String()
+
+    git_url = fields.String()
+    branch = fields.String()
+
+
 class ProjectMigrationCheckResponse(Schema):
     """Response schema for project migration check."""
 
     migration_required = fields.Boolean()
+    template_update_possible = fields.Boolean()
+    automated_template_update = fields.Boolean()
+    current_template_version = fields.String(allow_none=True)
+    latest_template_version = fields.String(allow_none=True)
+    template_source = fields.String()
+    template_ref = fields.String()
+    template_id = fields.String()
+    docker_update_possible = fields.Boolean()
     project_supported = fields.Boolean()
     project_version = fields.String()
     latest_version = fields.String()
