@@ -19,32 +19,21 @@
 import marshmallow
 from marshmallow import Schema, fields, post_load, pre_load
 
+from renku.core.models.datasets import DatasetCreatorsJson as DatasetCreators
+from renku.core.models.datasets import DatasetDetailsJson as DatasetDetails
+from renku.core.models.datasets import ImageObjectJson as ImageObject
+from renku.core.models.datasets import ImageObjectRequestJson as ImageObjectRequest
+from renku.service.serializers.common import RenkuSyncSchema
 from renku.service.serializers.rpc import JsonRPCResponse
 
 
-class DatasetCreators(Schema):
-    """Schema for the dataset creators."""
+class DatasetDetailsRequest(DatasetDetails):
+    """Request schema with dataset image information."""
 
-    name = fields.String()
-    email = fields.String()
-    affiliation = fields.String()
+    images = marshmallow.fields.List(marshmallow.fields.Nested(ImageObjectRequest))
 
 
-class DatasetDetails(Schema):
-    """Serialize a dataset to a response object."""
-
-    name = fields.String(required=True)
-    version = fields.String(allow_none=True)
-    created_at = fields.String(allow_none=True, attribute="date_created")
-
-    title = fields.String()
-    creators = fields.List(fields.Nested(DatasetCreators))
-    description = fields.String()
-    keywords = fields.List(fields.String())
-    identifier = fields.String()
-
-
-class DatasetCreateRequest(DatasetDetails):
+class DatasetCreateRequest(DatasetDetailsRequest):
     """Request schema for a dataset create view."""
 
     project_id = fields.String(required=True)
@@ -60,11 +49,10 @@ class DatasetCreateRequest(DatasetDetails):
         return data
 
 
-class DatasetCreateResponse(Schema):
+class DatasetCreateResponse(RenkuSyncSchema):
     """Response schema for a dataset create view."""
 
     name = fields.String(required=True)
-    remote_branch = fields.String()
 
 
 class DatasetCreateResponseRPC(JsonRPCResponse):
@@ -90,11 +78,10 @@ class DatasetRemoveRequest(DatasetDetails):
         return data
 
 
-class DatasetRemoveResponse(Schema):
+class DatasetRemoveResponse(RenkuSyncSchema):
     """Response schema for a dataset create view."""
 
     name = fields.String(required=True)
-    remote_branch = fields.String()
 
 
 class DatasetRemoveResponseRPC(JsonRPCResponse):
@@ -115,13 +102,13 @@ class DatasetAddFile(Schema):
 class DatasetAddRequest(Schema):
     """Request schema for a dataset add file view."""
 
-    project_id = fields.String(required=True)
     name = fields.String(required=True)
     files = fields.List(fields.Nested(DatasetAddFile), required=True)
 
     create_dataset = fields.Boolean(missing=False)
     force = fields.Boolean(missing=False)
 
+    project_id = fields.String(required=True)
     commit_message = fields.String()
     client_extras = fields.String()
 
@@ -143,14 +130,13 @@ class DatasetAddRequest(Schema):
         return data
 
 
-class DatasetAddResponse(Schema):
+class DatasetAddResponse(RenkuSyncSchema):
     """Response schema for a dataset add file view."""
 
     project_id = fields.String(required=True)
     name = fields.String(required=True)
 
     files = fields.List(fields.Nested(DatasetAddFile), required=True)
-    remote_branch = fields.String()
 
 
 class DatasetAddResponseRPC(JsonRPCResponse):
@@ -168,10 +154,16 @@ class DatasetListRequest(Schema):
     branch = fields.String()
 
 
+class DatasetDetailsResponse(DatasetDetails):
+    """Request schema with dataset image information."""
+
+    images = marshmallow.fields.List(marshmallow.fields.Nested(ImageObject))
+
+
 class DatasetListResponse(Schema):
     """Response schema for dataset list view."""
 
-    datasets = fields.List(fields.Nested(DatasetDetails), required=True)
+    datasets = fields.List(fields.Nested(DatasetDetailsResponse), required=True)
 
 
 class DatasetListResponseRPC(JsonRPCResponse):
@@ -239,17 +231,19 @@ class DatasetImportResponseRPC(JsonRPCResponse):
 class DatasetEditRequest(Schema):
     """Dataset edit metadata request."""
 
-    project_id = fields.String(required=True)
     name = fields.String(required=True)
-
     title = fields.String(default=None)
     description = fields.String(default=None)
     creators = fields.List(fields.Nested(DatasetCreators))
     keywords = fields.List(fields.String())
+    images = fields.List(fields.Nested(ImageObjectRequest))
+
+    project_id = fields.String()
+    git_url = fields.String()
     commit_message = fields.String()
 
 
-class DatasetEditResponse(Schema):
+class DatasetEditResponse(RenkuSyncSchema):
     """Dataset edit metadata response."""
 
     edited = fields.Dict(required=True)
@@ -265,12 +259,12 @@ class DatasetEditResponseRPC(JsonRPCResponse):
 class DatasetUnlinkRequest(Schema):
     """Dataset unlink file request."""
 
-    project_id = fields.String(required=True)
     name = fields.String(required=True)
-
     include_filters = fields.List(fields.String())
     exclude_filters = fields.List(fields.String())
 
+    project_id = fields.String()
+    git_url = fields.String()
     commit_message = fields.String()
 
     @post_load()
@@ -285,7 +279,7 @@ class DatasetUnlinkRequest(Schema):
         return data
 
 
-class DatasetUnlinkResponse(Schema):
+class DatasetUnlinkResponse(RenkuSyncSchema):
     """Dataset unlink files response."""
 
     unlinked = fields.List(fields.String())
