@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -16,6 +16,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Test ``migrate`` command."""
+import json
+import os
 from pathlib import Path
 
 import pytest
@@ -26,64 +28,6 @@ from renku.core.management.config import RENKU_HOME
 from renku.core.management.migrate import SUPPORTED_PROJECT_VERSION, get_migrations
 
 
-@pytest.mark.migration
-@pytest.mark.parametrize(
-    "command",
-    [
-        ["config", "key", "value"],
-        ["dataset", "create", "new"],
-        ["dataset", "add", "new", "README.md"],
-        ["dataset", "edit", "new"],
-        ["dataset", "unlink", "new"],
-        ["dataset", "rm", "new"],
-        ["dataset", "update"],
-        ["dataset", "export", "new", "zenodo"],
-        ["dataset", "import", "uri"],
-        ["dataset", "rm-tags", "news"],
-        ["log"],
-        ["mv", "news"],
-        ["rerun", "data"],
-        ["run", "echo"],
-        ["show", "inputs"],
-        ["show", "outputs"],
-        ["show", "siblings"],
-        ["status"],
-        ["update"],
-        ["workflow"],
-    ],
-)
-def test_commands_fail_on_old_repository(isolated_runner, old_repository_with_submodules, command):
-    """Test commands that fail on projects created by old version of renku."""
-    runner = isolated_runner
-    result = runner.invoke(cli, command)
-    assert 1 == result.exit_code, result.output
-    output = result.output
-    assert "Project version is outdated and a migration is required" in output
-
-
-@pytest.mark.migration
-@pytest.mark.parametrize(
-    "command",
-    [
-        ["clone", "uri"],
-        ["config", "key"],
-        ["dataset"],
-        ["dataset", "ls-files"],
-        ["dataset", "ls-tags", "new"],
-        ["doctor"],
-        ["githooks", "install"],
-        ["help"],
-        ["init"],
-        ["storage", "check"],
-    ],
-)
-def test_commands_work_on_old_repository(isolated_runner, old_repository_with_submodules, command):
-    """Test commands that do not require migration."""
-    runner = isolated_runner
-    result = runner.invoke(cli, command)
-    assert "a migration is required" not in result.output
-
-
 @pytest.mark.migration
 def test_migrate_datasets_with_old_repository(isolated_runner, old_project):
     """Test migrate on old repository."""
@@ -104,6 +48,38 @@ def test_migrate_project(isolated_runner, old_project):
     assert client.project.name
 
 
+@pytest.mark.migration
+@pytest.mark.parametrize("old_project", ["old-workflows-v0.10.0.git"], indirect=["old_project"])
+def test_migrate_duplicated_input_binding(isolated_runner, old_project):
+    """Check migrating CWLs with multiple outputs binding to the same input."""
+    result = isolated_runner.invoke(cli, ["migrate", "--no-commit"])
+
+    assert 0 == result.exit_code, result.output
+
+
+@pytest.mark.migration
+@pytest.mark.serial
+def test_migration_check(isolated_runner, project):
+    """Test migrate on old repository."""
+    result = isolated_runner.invoke(cli, ["migrationscheck"])
+    assert 0 == result.exit_code
+    output = json.loads(result.output)
+    assert output.keys() == {
+        "latest_version",
+        "project_version",
+        "migration_required",
+        "project_supported",
+        "template_update_possible",
+        "current_template_version",
+        "latest_template_version",
+        "template_source",
+        "template_ref",
+        "template_id",
+        "automated_update",
+        "docker_update_possible",
+    }
+
+
 @pytest.mark.migration
 def test_correct_path_migrated(isolated_runner, old_project):
     """Check if path on dataset files has been correctly migrated."""
@@ -165,8 +141,8 @@ def test_remove_committed_lock_file(isolated_runner, old_project):
     result = isolated_runner.invoke(cli, ["migrate"])
     assert 0 == result.exit_code
 
-    assert (repo_path / ".renku.lock").exists() is False
-    assert repo.is_dirty() is False
+    assert not (repo_path / ".renku.lock").exists()
+    assert not repo.is_dirty()
 
     ignored = (repo_path / ".gitignore").read_text()
     assert ".renku.lock" in ignored
@@ -238,7 +214,7 @@ def test_comprehensive_dataset_migration(isolated_runner, old_dataset_project):
     assert 0 == result.exit_code
     assert "OK" in result.output
 
-    client = LocalClient(path=old_dataset_project.working_dir)
+    client = old_dataset_project
 
     dataset = client.load_dataset("dataverse")
     assert dataset._id.endswith("/datasets/1d2ed1e4-3aeb-4f25-90b2-38084ee3d86c")
@@ -253,7 +229,7 @@ def test_comprehensive_dataset_migration(isolated_runner, old_dataset_project):
     assert "https://doi.org/10.7910/DVN/EV6KLF" == dataset.same_as.url
     assert "1" == dataset.tags[0].name
     assert "Tag 1 created by renku import" == dataset.tags[0].description
-    assert isinstance(dataset.license, dict)
+    assert isinstance(dataset.license, str)
     assert "https://creativecommons.org/publicdomain/zero/1.0/" in str(dataset.license)
 
     file_ = dataset.find_file("data/dataverse/copy.sh")
@@ -286,3 +262,133 @@ def test_comprehensive_dataset_migration(isolated_runner, old_dataset_project):
     assert "README.md" == file_.source
     assert file_.based_on is None
     assert file_.url.endswith("/projects/mohammad.alisafaee/old-datasets-v0.9.1/files/blob/README.md")
+
+
+@pytest.mark.migration
+def test_no_blank_node_after_dataset_migration(isolated_runner, old_dataset_project):
+    """Test migration of datasets with blank nodes creates IRI identifiers."""
+    assert 0 == isolated_runner.invoke(cli, ["migrate"]).exit_code
+
+    dataset = old_dataset_project.load_dataset("201901_us_flights_1")
+
+    assert not dataset.creators[0]._id.startswith("_:")
+    assert not dataset.same_as._id.startswith("_:")
+    assert not dataset.tags[0]._id.startswith("_:")
+    assert isinstance(dataset.license, str)
+
+
+@pytest.mark.migration
+def test_migrate_non_renku_repository(isolated_runner):
+    """Test migration prints proper message when run on non-renku repo."""
+    from git import Repo
+
+    Repo.init(".")
+    os.mkdir(".renku")
+
+    result = isolated_runner.invoke(cli, ["migrate"])
+
+    assert 0 == result.exit_code, result.output
+    assert "Warning: Not a renku project." in result.output
+    assert "No migrations required." in result.output
+
+
+@pytest.mark.migration
+def test_migrate_check_on_old_project(isolated_runner, old_repository_with_submodules):
+    """Test migration check on an old project."""
+    result = isolated_runner.invoke(cli, ["migrate", "--check"])
+
+    assert 3 == result.exit_code
+    assert "Project version is outdated and a migration is required" in result.output
+
+
+@pytest.mark.migration
+def test_migrate_check_on_unsupported_project(isolated_runner, unsupported_project):
+    """Test migration check on an unsupported project."""
+    result = isolated_runner.invoke(cli, ["migrate", "--check"])
+
+    assert 4 == result.exit_code
+    assert "Project is not supported by this version of Renku." in result.output
+
+
+@pytest.mark.migration
+def test_migrate_check_on_non_renku_repository(isolated_runner):
+    """Test migration check on non-renku repo."""
+    from git import Repo
+
+    Repo.init(".")
+    os.mkdir(".renku")
+
+    result = isolated_runner.invoke(cli, ["migrate", "--check"])
+
+    assert 0 == result.exit_code, result.output
+    assert "Warning: Not a renku project." in result.output
+
+
+@pytest.mark.migration
+@pytest.mark.parametrize(
+    "command",
+    [
+        ["config", "set", "key", "value"],
+        ["dataset", "create", "new"],
+        ["dataset", "add", "new", "README.md"],
+        ["dataset", "edit", "new"],
+        ["dataset", "unlink", "new"],
+        ["dataset", "rm", "new"],
+        ["dataset", "update"],
+        ["dataset", "export", "new", "zenodo"],
+        ["dataset", "import", "uri"],
+        ["dataset", "rm-tags", "news"],
+        ["log"],
+        ["mv", "news"],
+        ["rerun", "data"],
+        ["run", "echo"],
+        ["show", "inputs"],
+        ["show", "outputs"],
+        ["show", "siblings"],
+        ["status"],
+        ["update", "--all"],
+        ["workflow", "ls"],
+    ],
+)
+def test_commands_fail_on_old_repository(isolated_runner, old_repository_with_submodules, command):
+    """Test commands that fail on projects created by old version of renku."""
+    result = isolated_runner.invoke(cli, command)
+    assert 3 == result.exit_code, result.output
+    assert "Project version is outdated and a migration is required" in result.output
+
+
+@pytest.mark.migration
+@pytest.mark.parametrize(
+    "command",
+    [
+        ["clone", "uri"],
+        ["config", "show", "key"],
+        ["dataset"],
+        ["dataset", "ls-files"],
+        ["dataset", "ls-tags", "new"],
+        ["doctor"],
+        ["githooks", "install"],
+        ["help"],
+        ["init"],
+        ["storage", "check"],
+    ],
+)
+def test_commands_work_on_old_repository(isolated_runner, old_repository_with_submodules, command):
+    """Test commands that do not require migration."""
+    result = isolated_runner.invoke(cli, command)
+    assert "Project version is outdated and a migration is required" not in result.output
+
+
+def test_commit_hook_with_immutable_modified_files(runner, local_client, mocker, template_update):
+    """Test repository update from a template with modified local immutable files."""
+    from renku.core.utils.contexts import chdir
+
+    template_update(immutable_files=["README.md"])
+
+    with chdir(local_client.path):
+        result = runner.invoke(cli, ["check-immutable-template-files", "Dockerfile"])
+        assert result.exit_code == 0
+
+        result = runner.invoke(cli, ["check-immutable-template-files", "README.md"])
+        assert result.exit_code == 1
+        assert "README.md" in result.output
