@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2017-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -20,6 +20,7 @@
 from __future__ import absolute_import, print_function
 
 import contextlib
+import datetime
 import os
 import subprocess
 import sys
@@ -34,8 +35,8 @@ from renku import __version__
 from renku.cli import cli
 from renku.core.management.repository import DEFAULT_DATA_DIR as DATA_DIR
 from renku.core.management.storage import StorageApiMixin
+from renku.core.models.enums import ConfigFilter
 from renku.core.utils.contexts import chdir
-from tests.cli.test_init import INPUT, METADATA, TEMPLATE_ID
 
 
 def test_version(runner):
@@ -57,7 +58,7 @@ def test_run_from_non_root(runner, client, cwd):
     path = client.path / cwd
     path.mkdir(parents=True, exist_ok=True)
     with chdir(path):
-        result = runner.invoke(cli, ["dataset"])
+        result = runner.invoke(cli, ["dataset", "ls"])
         assert 0 == result.exit_code
         assert "Run CLI commands only from project's root" in result.output
 
@@ -65,7 +66,7 @@ def test_run_from_non_root(runner, client, cwd):
         assert 0 == result.exit_code
         assert "Run CLI commands only from project" not in result.output
 
-    result = runner.invoke(cli, ["dataset"])
+    result = runner.invoke(cli, ["dataset", "ls"])
     assert 0 == result.exit_code
     assert "Run CLI commands only from project's root" not in result.output
 
@@ -113,7 +114,8 @@ def test_git_pre_commit_hook(runner, project, capsys):
     repo.git.add("--all")
     with pytest.raises(git.HookExecutionError) as error:
         repo.index.commit("hello")
-        assert output.name in error.stdout
+
+    assert output.name in error.value.stdout
 
     result = runner.invoke(cli, ["githooks", "uninstall"])
     assert 0 == result.exit_code
@@ -121,6 +123,40 @@ def test_git_pre_commit_hook(runner, project, capsys):
     repo.index.commit("hello")
 
 
+def test_git_pre_commit_hook_in_old_project(isolated_runner, old_dataset_project):
+    """Test proper messaging in git hooks when project requires migration."""
+    assert 0 == isolated_runner.invoke(cli, ["githooks", "install"]).exit_code
+
+    with old_dataset_project.with_metadata() as project:
+        project.created = datetime.datetime.now(datetime.timezone.utc)
+
+    old_dataset_project.repo.git.add("--all")
+
+    with pytest.raises(git.exc.HookExecutionError) as e:
+        old_dataset_project.repo.index.commit("update project metadata")
+
+    assert "Cannot verify validity of the commit: Project metadata is outdated." in str(e.value.stdout)
+    assert "Run 'renku migrate' command to fix the issue." in str(e.value.stdout)
+    assert "You are trying to update generated files" not in str(e.value.stdout)
+
+
+def test_git_pre_commit_hook_in_unsupported_project(unsupported_project):
+    """Test proper messaging in git hooks when project version is not supported."""
+    with unsupported_project.with_metadata() as project:
+        project.created = datetime.datetime.now(datetime.timezone.utc)
+
+    unsupported_project.repo.git.add("--all")
+
+    with pytest.raises(git.exc.HookExecutionError) as e:
+        unsupported_project.repo.index.commit("update project metadata")
+
+    assert "Cannot verify validity of the commit: Project was created with a newer version of Renku." in str(
+        e.value.stdout
+    )
+    assert "Upgrade Renku to the latest version." in str(e.value.stdout)
+    assert "You are trying to update generated files" not in str(e.value.stdout)
+
+
 def test_workflow(runner, project):
     """Test workflow command."""
     result = runner.invoke(cli, ["run", "touch", "data.csv"])
@@ -259,15 +295,15 @@ def test_streams_and_args_names(runner, project, capsys, no_lfs_warning):
     assert 0 == result.exit_code
 
 
-def test_show_inputs(tmpdir_factory, project, runner, run):
+def test_show_inputs(tmpdir_factory, project, runner, run, template):
     """Test show inputs with submodules."""
     second_project = Path(str(tmpdir_factory.mktemp("second_project")))
 
     parameters = []
-    for key in set(METADATA.keys()):
+    for key in set(template["metadata"].keys()):
         parameters.append("--parameter")
-        parameters.append(f'{key}="{METADATA[key]}"')
-    assert 0 == run(args=("init", str(second_project), "--template-id", TEMPLATE_ID, *parameters))
+        parameters.append(f'{key}="{template["metadata"][key]}"')
+    assert 0 == run(args=("init", str(second_project), "--template-id", template["id"], *parameters))
 
     woop = second_project / "woop"
     with woop.open("w") as fp:
@@ -290,14 +326,15 @@ def test_show_inputs(tmpdir_factory, project, runner, run):
     assert {str(imported_woop.resolve().relative_to(Path(project).resolve()))} == set(result.output.strip().split("\n"))
 
 
-def test_configuration_of_no_external_storage(isolated_runner, monkeypatch):
+def test_configuration_of_no_external_storage(isolated_runner, monkeypatch, project_init):
     """Test the LFS requirement for renku run with disabled storage."""
     runner = isolated_runner
+    data, commands = project_init
 
     os.mkdir("test-project")
     os.chdir("test-project")
 
-    result = runner.invoke(cli, ["--no-external-storage", "init", ".", "--template-id", TEMPLATE_ID], INPUT)
+    result = runner.invoke(cli, ["--no-external-storage"] + commands["init"] + commands["id"], commands["confirm"])
     assert 0 == result.exit_code
     # Pretend that git-lfs is not installed.
     with monkeypatch.context() as monkey:
@@ -318,11 +355,12 @@ def test_configuration_of_no_external_storage(isolated_runner, monkeypatch):
     assert 1 == result.exit_code
 
 
-def test_configuration_of_external_storage(isolated_runner, monkeypatch):
+def test_configuration_of_external_storage(isolated_runner, monkeypatch, project_init):
     """Test the LFS requirement for renku run."""
     runner = isolated_runner
+    data, commands = project_init
 
-    result = runner.invoke(cli, ["--external-storage", "init", ".", "--template-id", TEMPLATE_ID], INPUT)
+    result = runner.invoke(cli, ["--external-storage"] + commands["init"] + commands["id"], commands["confirm"])
     assert 0 == result.exit_code
     # Pretend that git-lfs is not installed.
     with monkeypatch.context() as monkey:
@@ -342,9 +380,13 @@ def test_configuration_of_external_storage(isolated_runner, monkeypatch):
     assert 0 == result.exit_code
 
 
-def test_early_check_of_external_storage(isolated_runner, monkeypatch, directory_tree):
+def test_early_check_of_external_storage(isolated_runner, monkeypatch, directory_tree, project_init):
     """Test LFS is checked early."""
-    result = isolated_runner.invoke(cli, ["--no-external-storage", "init", ".", "--template-id", TEMPLATE_ID])
+    data, commands = project_init
+
+    result = isolated_runner.invoke(
+        cli, ["--no-external-storage"] + commands["init"] + commands["id"], commands["confirm"]
+    )
     assert 0 == result.exit_code
 
     result = isolated_runner.invoke(cli, ["dataset", "create", "my-dataset"])
@@ -354,7 +396,7 @@ def test_early_check_of_external_storage(isolated_runner, monkeypatch, directory
     with monkeypatch.context() as monkey:
         monkey.setattr(StorageApiMixin, "storage_installed", False)
 
-        failing_command = ["dataset", "add", "-s", "src", "my-dataset", directory_tree.strpath]
+        failing_command = ["dataset", "add", "-s", "src", "my-dataset", str(directory_tree)]
         result = isolated_runner.invoke(cli, failing_command)
         assert 1 == result.exit_code
         assert "External storage is not configured" in result.output
@@ -364,15 +406,16 @@ def test_early_check_of_external_storage(isolated_runner, monkeypatch, directory
         assert 'Cannot use "--source" with URLs' in result.output
 
 
-def test_file_tracking(isolated_runner):
+def test_file_tracking(isolated_runner, project_init):
     """Test .gitattribute handling on renku run."""
     runner = isolated_runner
+    data, commands = project_init
 
     os.mkdir("test-project")
     os.chdir("test-project")
-    result = runner.invoke(cli, ["init", ".", "--template-id", TEMPLATE_ID], INPUT)
+    result = runner.invoke(cli, commands["init"] + commands["id"], commands["confirm"])
     assert 0 == result.exit_code
-    result = runner.invoke(cli, ["config", "lfs_threshold", "0b"])
+    result = runner.invoke(cli, ["config", "set", "lfs_threshold", "0b"])
     assert 0 == result.exit_code
 
     result = runner.invoke(cli, ["run", "touch", "tracked"])
@@ -385,9 +428,10 @@ def test_file_tracking(isolated_runner):
 
 
 @pytest.mark.xfail
-def test_status_with_submodules(isolated_runner, monkeypatch):
+def test_status_with_submodules(isolated_runner, monkeypatch, project_init):
     """Test status calculation with submodules."""
     runner = isolated_runner
+    data, commands = project_init
 
     os.mkdir("foo")
     os.mkdir("bar")
@@ -397,13 +441,13 @@ def test_status_with_submodules(isolated_runner, monkeypatch):
 
     os.chdir("foo")
     result = runner.invoke(
-        cli, ["init", ".", "--template", TEMPLATE_ID, "--no-external-storage"], INPUT, catch_exceptions=False
+        cli, commands["init"] + commands["id"] + ["--no-external-storage"], commands["confirm"], catch_exceptions=False
     )
     assert 0 == result.exit_code
 
     os.chdir("../bar")
     result = runner.invoke(
-        cli, ["init", ".", "--template", TEMPLATE_ID, "--no-external-storage"], INPUT, catch_exceptions=False
+        cli, commands["init"] + commands["id"] + ["--no-external-storage"], commands["confirm"], catch_exceptions=False
     )
     assert 0 == result.exit_code
 
@@ -719,7 +763,7 @@ def test_config_manager_creation(client, global_config_dir, global_only, config_
     """Check creation of configuration file."""
     path = getattr(client, config_path_attr)
     assert path.endswith("renku.ini")
-    config = client.load_config(local_only=False, global_only=False)
+    config = client.load_config(config_filter=ConfigFilter.ALL)
     client.store_config(config, global_only=global_only)
     assert Path(path).exists()
 
@@ -727,15 +771,18 @@ def test_config_manager_creation(client, global_config_dir, global_only, config_
 @pytest.mark.parametrize("global_only", (False, True))
 def test_config_manager_set_value(client, global_config_dir, global_only):
     """Check writing to configuration."""
-    local_only = not global_only
+    config_filter = ConfigFilter.GLOBAL_ONLY
+
+    if not global_only:
+        config_filter = ConfigFilter.LOCAL_ONLY
 
     client.set_value("zenodo", "access_token", "my-secret", global_only=global_only)
 
-    config = client.load_config(local_only=local_only, global_only=global_only)
+    config = client.load_config(config_filter=config_filter)
     assert config.get("zenodo", "access_token") == "my-secret"
 
     client.remove_value("zenodo", "access_token", global_only=global_only)
-    config = client.load_config(local_only=local_only, global_only=global_only)
+    config = client.load_config(config_filter=config_filter)
     assert "zenodo" not in config.sections()
 
 
@@ -745,14 +792,14 @@ def test_config_get_value(client, global_config_dir):
     client.set_value("local", "key", "local-value")
     value = client.get_value("local", "key")
     assert "local-value" == value
-    value = client.get_value("local", "key", global_only=True)
+    value = client.get_value("local", "key", config_filter=ConfigFilter.GLOBAL_ONLY)
     assert value is None
 
     # Value set globally is stored globally
     client.set_value("global", "key", "global-value", global_only=True)
-    value = client.get_value("global", "key", local_only=True)
+    value = client.get_value("global", "key", config_filter=ConfigFilter.LOCAL_ONLY)
     assert value is None
-    value = client.get_value("global", "key", global_only=True)
+    value = client.get_value("global", "key", config_filter=ConfigFilter.GLOBAL_ONLY)
     assert "global-value" == value
     value = client.get_value("global", "key")
     assert "global-value" == value
@@ -762,13 +809,14 @@ def test_config_get_value(client, global_config_dir):
     assert value is None
 
 
-def test_lfs_size_limit(isolated_runner):
+def test_lfs_size_limit(isolated_runner, project_init):
     """Test inclusion of files in lfs by size."""
     runner = isolated_runner
+    data, commands = project_init
 
     os.mkdir("test-project")
     os.chdir("test-project")
-    result = runner.invoke(cli, ["init", ".", "--template-id", TEMPLATE_ID], INPUT)
+    result = runner.invoke(cli, commands["init"] + commands["id"], commands["confirm"])
     assert 0 == result.exit_code
 
     large = Path("large")
@@ -802,15 +850,16 @@ def test_lfs_size_limit(isolated_runner):
         ("dir2\n!*.test", "dir2/file.test", True),
     ),
 )
-def test_lfs_ignore(isolated_runner, ignore, path, tracked):
+def test_lfs_ignore(isolated_runner, ignore, path, tracked, project_init):
     """Test inclusion of files in lfs by size."""
     runner = isolated_runner
+    data, commands = project_init
 
     os.mkdir("test-project")
     os.chdir("test-project")
-    result = runner.invoke(cli, ["init", ".", "--template-id", TEMPLATE_ID], INPUT)
+    result = runner.invoke(cli, commands["init"] + commands["id"], commands["confirm"])
     assert 0 == result.exit_code
-    result = runner.invoke(cli, ["config", "lfs_threshold", "0b"])
+    result = runner.invoke(cli, ["config", "set", "lfs_threshold", "0b"])
     assert 0 == result.exit_code
 
     with Path(".renkulfsignore").open("w") as f:
