@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2017-2020- Swiss Data Science Center (SDSC)
+# Copyright 2017-2021- Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -20,53 +20,14 @@
 import datetime
 from urllib.parse import urljoin
 
-import yaml
-
 from renku.core.management.client import LocalClient
 from renku.core.models.datasets import Dataset
 from renku.core.utils.uuid import is_uuid
 
 
-def test_dataset_serialization(client, dataset, data_file):
-    """Test Dataset serialization."""
-
-    def load_dataset(name):
-        with open(str(client.get_dataset_path(name))) as f:
-            return yaml.safe_load(f)
-
-    d_dict = load_dataset("dataset")
-
-    expected_fields = [
-        "@id",
-        "http://www.w3.org/2000/01/rdf-schema#label",
-        "http://schema.org/isPartOf",
-        "http://schema.org/dateCreated",
-        "http://schema.org/creator",
-        "http://schema.org/datePublished",
-        "http://schema.org/description",
-        "http://schema.org/hasPart",
-        "http://schema.org/identifier",
-        "http://schema.org/inLanguage",
-        "http://schema.org/keywords",
-        "http://schema.org/license",
-        "http://schema.org/name",
-        "http://www.w3.org/ns/prov#atLocation",
-        "http://schema.org/url",
-        "http://schema.org/version",
-    ]
-    for field in expected_fields:
-        assert field in d_dict
-
-    assert not d_dict["http://schema.org/hasPart"]
-    client.add_data_to_dataset(dataset, [str(data_file)])
-    dataset.to_yaml()
-    d_dict = load_dataset("dataset")
-    assert d_dict["http://schema.org/hasPart"]
-
-
-def test_dataset_deserialization(client, dataset):
+def test_dataset_deserialization(client_with_datasets):
     """Test Dataset deserialization."""
-    dataset_ = Dataset.from_yaml(client.get_dataset_path("dataset"), client=client)
+    dataset = Dataset.from_yaml(client_with_datasets.get_dataset_path("dataset-1"), client=client_with_datasets)
 
     dataset_types = {
         "date_created": [datetime.datetime],
@@ -78,11 +39,11 @@ def test_dataset_deserialization(client, dataset):
     }
 
     for attribute, type_ in dataset_types.items():
-        assert type(dataset_.__getattribute__(attribute)) in type_
+        assert type(dataset.__getattribute__(attribute)) in type_
 
     creator_types = {"email": str, "_id": str, "name": str, "affiliation": str}
 
-    creator = dataset.creators[0]
+    creator = client_with_datasets.load_dataset("dataset-1").creators[0]
 
     for attribute, type_ in creator_types.items():
         assert type(getattr(creator, attribute)) is type_
@@ -126,3 +87,17 @@ def test_calamus(client, dataset_metadata_before_calamus):
     file_ = dataset.find_file("data/dataverse/local/result.csv")
     assert file_.external is False
     assert "file://../../../../tmp/result.csv" == file_.url
+
+
+def test_dataset_with_multiple_project_version(client_with_datasets):
+    """Test deserialization of a dataset where contains different project versions."""
+    max_version = "42000"
+
+    # Change project version for a single file
+    with client_with_datasets.with_dataset("dataset-2") as dataset:
+        file_ = dataset.find_file(dataset.data_dir / "file1")
+        file_._project.version = max_version
+
+    dataset = client_with_datasets.load_dataset("dataset-2")
+
+    assert {max_version} == {f._project.version for f in dataset.files}
