@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2020 - Swiss Data Science Center (SDSC)
+# Copyright 2020-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -19,41 +19,55 @@
 import json
 import uuid
 
+import jwt
 import pytest
 from flaky import flaky
 from git import Repo
+from werkzeug.utils import secure_filename
 
 from renku.core.errors import DatasetExistsError, ParameterError
 from renku.service.jobs.cleanup import cache_project_cleanup
 from renku.service.jobs.datasets import dataset_add_remote_file, dataset_import
+from renku.service.serializers.headers import JWT_TOKEN_SECRET, encode_b64
 from renku.service.utils import make_project_path
 from tests.service.views.test_dataset_views import assert_rpc_response
 
 
-@pytest.mark.parametrize("url", [("https://dev.renku.ch/datasets/428c3626-1c56-463d-8753-336470cc6917/")])
+@pytest.mark.parametrize("url", ["https://dev.renku.ch/datasets/428c3626-1c56-463d-8753-336470cc6917/"])
 @pytest.mark.integration
 @flaky(max_runs=30, min_passes=1)
 def test_dataset_url_import_job(url, svc_client_with_repo):
     """Test dataset import via url."""
     svc_client, headers, project_id, url_components = svc_client_with_repo
-    user = {"user_id": headers["Renku-User-Id"]}
+
+    decoded = jwt.decode(headers["Renku-User"], JWT_TOKEN_SECRET, algorithms=["HS256"], audience="renku",)
+    user_data = {
+        "fullname": decoded["name"],
+        "email": decoded["email"],
+        "user_id": encode_b64(secure_filename(decoded["sub"])),
+        "token": headers["Authorization"].split("Bearer ")[-1],
+    }
+
     payload = {
         "project_id": project_id,
         "dataset_uri": url,
     }
+
     response = svc_client.post("/datasets.import", data=json.dumps(payload), headers=headers,)
 
     assert response
     assert_rpc_response(response)
     assert {"job_id", "created_at"} == set(response.json["result"].keys())
 
-    dest = make_project_path(user, {"owner": url_components.owner, "name": url_components.name})
+    dest = make_project_path(
+        user_data, {"owner": url_components.owner, "name": url_components.name, "project_id": project_id}
+    )
 
     old_commit = Repo(dest).head.commit
     job_id = response.json["result"]["job_id"]
 
     dataset_import(
-        user, job_id, project_id, url,
+        user_data, job_id, project_id, url,
     )
 
     new_commit = Repo(dest).head.commit
@@ -74,7 +88,10 @@ def test_dataset_url_import_job(url, svc_client_with_repo):
 def test_dataset_import_job(doi, svc_client_with_repo):
     """Test dataset import via doi."""
     svc_client, headers, project_id, url_components = svc_client_with_repo
-    user = {"user_id": headers["Renku-User-Id"]}
+
+    user_id = encode_b64(secure_filename("9ab2fc80-3a5c-426d-ae78-56de01d214df"))
+    user = {"user_id": user_id}
+
     payload = {
         "project_id": project_id,
         "dataset_uri": doi,
@@ -85,7 +102,9 @@ def test_dataset_import_job(doi, svc_client_with_repo):
     assert_rpc_response(response)
     assert {"job_id", "created_at"} == set(response.json["result"].keys())
 
-    dest = make_project_path(user, {"owner": url_components.owner, "name": url_components.name})
+    dest = make_project_path(
+        user, {"owner": url_components.owner, "name": url_components.name, "project_id": project_id}
+    )
 
     old_commit = Repo(dest).head.commit
     job_id = response.json["result"]["job_id"]
@@ -119,7 +138,10 @@ def test_dataset_import_job(doi, svc_client_with_repo):
 def test_dataset_import_junk_job(doi, expected_err, svc_client_with_repo):
     """Test dataset import."""
     svc_client, headers, project_id, url_components = svc_client_with_repo
-    user = {"user_id": headers["Renku-User-Id"]}
+
+    user_id = encode_b64(secure_filename("9ab2fc80-3a5c-426d-ae78-56de01d214df"))
+    user = {"user_id": user_id}
+
     payload = {
         "project_id": project_id,
         "dataset_uri": doi,
@@ -130,7 +152,9 @@ def test_dataset_import_junk_job(doi, expected_err, svc_client_with_repo):
     assert_rpc_response(response)
     assert {"job_id", "created_at"} == set(response.json["result"].keys())
 
-    dest = make_project_path(user, {"owner": url_components.owner, "name": url_components.name})
+    dest = make_project_path(
+        user, {"owner": url_components.owner, "name": url_components.name, "project_id": project_id}
+    )
 
     old_commit = Repo(dest).head.commit
     job_id = response.json["result"]["job_id"]
@@ -159,7 +183,10 @@ def test_dataset_import_junk_job(doi, expected_err, svc_client_with_repo):
 def test_dataset_import_twice_job(doi, svc_client_with_repo):
     """Test dataset import."""
     svc_client, headers, project_id, url_components = svc_client_with_repo
-    user = {"user_id": headers["Renku-User-Id"]}
+
+    user_id = encode_b64(secure_filename("9ab2fc80-3a5c-426d-ae78-56de01d214df"))
+    user = {"user_id": user_id}
+
     payload = {
         "project_id": project_id,
         "dataset_uri": doi,
@@ -170,7 +197,9 @@ def test_dataset_import_twice_job(doi, svc_client_with_repo):
     assert_rpc_response(response)
     assert {"job_id", "created_at"} == set(response.json["result"].keys())
 
-    dest = make_project_path(user, {"owner": url_components.owner, "name": url_components.name})
+    dest = make_project_path(
+        user, {"owner": url_components.owner, "name": url_components.name, "project_id": project_id}
+    )
 
     old_commit = Repo(dest).head.commit
     job_id = response.json["result"]["job_id"]
@@ -208,16 +237,20 @@ def test_dataset_import_twice_job(doi, svc_client_with_repo):
 def test_dataset_add_remote_file(url, svc_client_with_repo):
     """Test dataset add a remote file."""
     svc_client, headers, project_id, url_components = svc_client_with_repo
-    user = {"user_id": headers["Renku-User-Id"]}
+
+    user_id = encode_b64(secure_filename("9ab2fc80-3a5c-426d-ae78-56de01d214df"))
+    user = {"user_id": user_id}
 
     payload = {"project_id": project_id, "name": uuid.uuid4().hex, "create_dataset": True, "files": [{"file_url": url}]}
     response = svc_client.post("/datasets.add", data=json.dumps(payload), headers=headers,)
 
     assert response
     assert_rpc_response(response)
-    assert {"files", "name", "project_id"} == set(response.json["result"].keys())
+    assert {"files", "name", "project_id", "remote_branch"} == set(response.json["result"].keys())
 
-    dest = make_project_path(user, {"owner": url_components.owner, "name": url_components.name})
+    dest = make_project_path(
+        user, {"owner": url_components.owner, "name": url_components.name, "project_id": project_id}
+    )
     old_commit = Repo(dest).head.commit
     job_id = response.json["result"]["files"][0]["job_id"]
     commit_message = "service: dataset add remote file"
@@ -236,7 +269,9 @@ def test_dataset_add_remote_file(url, svc_client_with_repo):
 def test_dataset_project_lock(doi, svc_client_with_repo):
     """Test dataset project lock."""
     svc_client, headers, project_id, url_components = svc_client_with_repo
-    user = {"user_id": headers["Renku-User-Id"]}
+    user_id = encode_b64(secure_filename("9ab2fc80-3a5c-426d-ae78-56de01d214df"))
+    user = {"user_id": user_id}
+
     payload = {
         "project_id": project_id,
         "dataset_uri": doi,
@@ -247,7 +282,9 @@ def test_dataset_project_lock(doi, svc_client_with_repo):
     assert_rpc_response(response)
     assert {"job_id", "created_at"} == set(response.json["result"].keys())
 
-    dest = make_project_path(user, {"owner": url_components.owner, "name": url_components.name})
+    dest = make_project_path(
+        user, {"owner": url_components.owner, "name": url_components.name, "project_id": project_id}
+    )
 
     old_commit = Repo(dest).head.commit
 
