@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2019-2020 - Swiss Data Science Center (SDSC)
+# Copyright 2019-2021 - Swiss Data Science Center (SDSC)
 # A partnership between École Polytechnique Fédérale de Lausanne (EPFL) and
 # Eidgenössische Technische Hochschule Zürich (ETHZ).
 #
@@ -16,16 +16,19 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 """Renku service cache view tests."""
+import copy
 import io
 import json
 import uuid
 
+import jwt
 import pytest
 from flaky import flaky
+from git import Repo
 
-from conftest import IT_GIT_ACCESS_TOKEN, IT_REMOTE_REPO_URL
 from renku.core.models.git import GitURL
-from renku.service.config import INVALID_HEADERS_ERROR_CODE, INVALID_PARAMS_ERROR_CODE
+from renku.service.config import INVALID_HEADERS_ERROR_CODE, RENKU_EXCEPTION_ERROR_CODE
+from renku.service.serializers.headers import JWT_TOKEN_SECRET
 
 
 @pytest.mark.service
@@ -42,10 +45,9 @@ def test_serve_api_spec(svc_client):
 
 
 @pytest.mark.service
-def test_list_upload_files_all(svc_client):
+def test_list_upload_files_all(svc_client, identity_headers):
     """Check list uploaded files view."""
-    headers_user = {"Content-Type": "application/json", "accept": "application/json", "Renku-User-Id": "user"}
-    response = svc_client.get("/cache.files_list", headers=headers_user)
+    response = svc_client.get("/cache.files_list", headers=identity_headers)
 
     assert {"result"} == set(response.json.keys())
 
@@ -69,12 +71,13 @@ def test_list_upload_files_all_no_auth(svc_client):
 
 
 @pytest.mark.service
-def test_file_upload(svc_client):
+def test_file_upload(svc_client, identity_headers):
     """Check successful file upload."""
-    headers_user = {"Renku-User-Id": "{0}".format(uuid.uuid4().hex)}
+    headers = copy.deepcopy(identity_headers)
+    headers.pop("Content-Type")
 
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile.txt"),), headers=headers_user,
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), uuid.uuid4().hex),), headers=headers,
     )
 
     assert response
@@ -85,37 +88,39 @@ def test_file_upload(svc_client):
 
 
 @pytest.mark.service
-def test_file_upload_override(svc_client):
+def test_file_upload_override(svc_client, identity_headers):
     """Check successful file upload."""
-    headers_user = {"Renku-User-Id": "{0}".format(uuid.uuid4().hex)}
+    headers = copy.deepcopy(identity_headers)
+    headers.pop("Content-Type")
+
+    filename = uuid.uuid4().hex
 
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile.txt"),), headers=headers_user,
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), filename),), headers=headers,
     )
 
     assert response
     assert 200 == response.status_code
-
     assert {"result"} == set(response.json.keys())
     assert isinstance(uuid.UUID(response.json["result"]["files"][0]["file_id"]), uuid.UUID)
     old_file_id = response.json["result"]["files"][0]["file_id"]
 
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile.txt"),), headers=headers_user,
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), filename),), headers=headers,
     )
 
     assert response
     assert 200 == response.status_code
 
     assert {"error"} == set(response.json.keys())
-    assert INVALID_PARAMS_ERROR_CODE == response.json["error"]["code"]
+    assert RENKU_EXCEPTION_ERROR_CODE == response.json["error"]["code"]
     assert "file exists" == response.json["error"]["reason"]
 
     response = svc_client.post(
         "/cache.files_upload",
-        data=dict(file=(io.BytesIO(b"this is a test"), "datafile.txt"),),
+        data=dict(file=(io.BytesIO(b"this is a test"), filename),),
         query_string={"override_existing": True},
-        headers=headers_user,
+        headers=headers,
     )
 
     assert response
@@ -127,11 +132,15 @@ def test_file_upload_override(svc_client):
 
 
 @pytest.mark.service
-def test_file_upload_same_file(svc_client):
+def test_file_upload_same_file(svc_client, identity_headers):
     """Check successful file upload with same file uploaded twice."""
-    headers_user1 = {"Renku-User-Id": "{0}".format(uuid.uuid4().hex)}
+    headers = copy.deepcopy(identity_headers)
+    headers.pop("Content-Type")
+
+    filename = uuid.uuid4().hex
+
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile.txt"),), headers=headers_user1,
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), filename),), headers=headers,
     )
 
     assert response
@@ -142,13 +151,13 @@ def test_file_upload_same_file(svc_client):
     assert isinstance(uuid.UUID(response.json["result"]["files"][0]["file_id"]), uuid.UUID)
 
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile.txt"),), headers=headers_user1,
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), filename),), headers=headers,
     )
 
     assert response
     assert 200 == response.status_code
     assert {"error"} == set(response.json.keys())
-    assert INVALID_PARAMS_ERROR_CODE == response.json["error"]["code"]
+    assert RENKU_EXCEPTION_ERROR_CODE == response.json["error"]["code"]
     assert "file exists" == response.json["error"]["reason"]
 
 
@@ -165,13 +174,31 @@ def test_file_upload_no_auth(svc_client):
 
 
 @pytest.mark.service
-def test_file_upload_with_users(svc_client):
+def test_file_upload_with_users(svc_client, identity_headers):
     """Check successful file upload and listing based on user auth header."""
-    headers_user1 = {"Renku-User-Id": "{0}".format(uuid.uuid4().hex)}
-    headers_user2 = {"Renku-User-Id": "{0}".format(uuid.uuid4().hex)}
+    headers_user1 = copy.deepcopy(identity_headers)
+    headers_user1.pop("Content-Type")
+
+    filename = uuid.uuid4().hex
+
+    jwt_data = {
+        "aud": ["renku"],
+        "email_verified": False,
+        "preferred_username": "user1@platform2.com",
+        "given_name": "user",
+        "family_name": "user one",
+        "name": "User One",
+        "email": "user1@platform2.com",
+        "sub": "8d1f08e2-b136-4c93-a38f-d5f36a5919d9",
+    }
+
+    headers_user2 = {
+        "Renku-User": jwt.encode(jwt_data, JWT_TOKEN_SECRET, algorithm="HS256"),
+        "Authorization": identity_headers["Authorization"],
+    }
 
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile1.txt"),), headers=headers_user1
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), filename),), headers=headers_user1
     )
 
     assert {"result"} == set(response.json.keys())
@@ -181,7 +208,7 @@ def test_file_upload_with_users(svc_client):
     assert 200 == response.status_code
 
     response = svc_client.post(
-        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), "datafile1.txt"),), headers=headers_user2
+        "/cache.files_upload", data=dict(file=(io.BytesIO(b"this is a test"), filename),), headers=headers_user2
     )
 
     assert response
@@ -192,20 +219,18 @@ def test_file_upload_with_users(svc_client):
     assert response
 
     assert {"result"} == set(response.json.keys())
-    assert 1 == len(response.json["result"]["files"])
+    assert 0 < len(response.json["result"]["files"])
 
-    file = response.json["result"]["files"][0]
-    assert file_id == file["file_id"]
-    assert 0 < file["file_size"]
+    assert file_id in [file["file_id"] for file in response.json["result"]["files"]]
 
 
 @pytest.mark.service
 @pytest.mark.integration
 @flaky(max_runs=10, min_passes=1)
-def test_clone_projects_no_auth(svc_client):
+def test_clone_projects_no_auth(svc_client, identity_headers, it_remote_repo_url):
     """Check error on cloning of remote repository."""
     payload = {
-        "git_url": IT_REMOTE_REPO_URL,
+        "git_url": it_remote_repo_url,
     }
 
     response = svc_client.post(
@@ -218,16 +243,7 @@ def test_clone_projects_no_auth(svc_client):
     err_message = "user identification is incorrect or missing"
     assert err_message == response.json["error"]["reason"]
 
-    headers = {
-        "Content-Type": "application/json",
-        "accept": "application/json",
-        "Renku-User-Id": "{0}".format(uuid.uuid4().hex),
-        "Renku-User-FullName": "Just Sam",
-        "Renku-User-Email": "contact@justsam.io",
-        "Authorization": f"Bearer {IT_GIT_ACCESS_TOKEN}",
-    }
-
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
     assert response
     assert {"result"} == set(response.json.keys())
 
@@ -235,71 +251,55 @@ def test_clone_projects_no_auth(svc_client):
 @pytest.mark.service
 @pytest.mark.integration
 @flaky(max_runs=10, min_passes=1)
-def test_clone_projects_with_auth(svc_client):
+def test_clone_projects_with_auth(svc_client, identity_headers, it_remote_repo_url):
     """Check cloning of remote repository."""
-    headers = {
-        "Content-Type": "application/json",
-        "accept": "application/json",
-        "Renku-User-Id": "{0}".format(uuid.uuid4().hex),
-        "Renku-User-FullName": "Just Sam",
-        "Renku-User-Email": "contact@justsam.io",
-        "Authorization": "Bearer {0}".format(IT_GIT_ACCESS_TOKEN),
-    }
-
     payload = {
-        "git_url": IT_REMOTE_REPO_URL,
+        "git_url": it_remote_repo_url,
     }
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
 
     assert response
     assert {"result"} == set(response.json.keys())
+    assert response.json["result"]["initialized"]
 
 
 @pytest.mark.service
 @pytest.mark.integration
 @flaky(max_runs=10, min_passes=1)
-def test_clone_projects_multiple(svc_client):
+def test_clone_projects_multiple(svc_client, identity_headers, it_remote_repo_url):
     """Check multiple cloning of remote repository."""
     project_ids = []
 
-    headers = {
-        "Content-Type": "application/json",
-        "Renku-User-Id": "{0}".format(uuid.uuid4().hex),
-        "Renku-User-FullName": "Just Sam",
-        "Renku-User-Email": "contact@justsam.io",
-        "Authorization": "Bearer {0}".format(IT_GIT_ACCESS_TOKEN),
-    }
-
     payload = {
-        "git_url": IT_REMOTE_REPO_URL,
+        "git_url": it_remote_repo_url,
     }
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
     assert response
 
     assert {"result"} == set(response.json.keys())
     project_ids.append(response.json["result"])
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
 
     assert response
     assert {"result"} == set(response.json.keys())
     project_ids.append(response.json["result"])
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
 
     assert response
     assert {"result"} == set(response.json.keys())
     project_ids.append(response.json["result"])
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
 
     assert response
     assert {"result"} == set(response.json.keys())
     last_pid = response.json["result"]["project_id"]
 
-    response = svc_client.get("/cache.project_list", headers=headers)
+    response = svc_client.get("/cache.project_list", headers=identity_headers)
 
     assert response
     assert {"result"} == set(response.json.keys())
@@ -314,21 +314,13 @@ def test_clone_projects_multiple(svc_client):
 @pytest.mark.service
 @pytest.mark.integration
 @flaky(max_runs=10, min_passes=1)
-def test_clone_projects_list_view_errors(svc_client):
+def test_clone_projects_list_view_errors(svc_client, identity_headers, it_remote_repo_url):
     """Check cache state of cloned projects with no headers."""
-    headers = {
-        "Content-Type": "application/json",
-        "Renku-User-Id": "{0}".format(uuid.uuid4().hex),
-        "Renku-User-FullName": "Just Sam",
-        "Renku-User-Email": "contact@justsam.io",
-        "Authorization": "Bearer {0}".format(IT_GIT_ACCESS_TOKEN),
-    }
-
     payload = {
-        "git_url": IT_REMOTE_REPO_URL,
+        "git_url": it_remote_repo_url,
     }
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers)
     assert response
     assert {"result"} == set(response.json.keys())
 
@@ -343,7 +335,7 @@ def test_clone_projects_list_view_errors(svc_client):
     assert {"error"} == set(response.json.keys())
     assert INVALID_HEADERS_ERROR_CODE == response.json["error"]["code"]
 
-    response = svc_client.get("/cache.project_list", headers=headers)
+    response = svc_client.get("/cache.project_list", headers=identity_headers)
 
     assert response
     assert {"result"} == set(response.json.keys())
@@ -357,23 +349,15 @@ def test_clone_projects_list_view_errors(svc_client):
 @pytest.mark.service
 @pytest.mark.integration
 @flaky(max_runs=10, min_passes=1)
-def test_clone_projects_invalid_headers(svc_client):
+def test_clone_projects_invalid_headers(svc_client, identity_headers, it_remote_repo_url):
     """Check cache state of cloned projects with invalid headers."""
-    headers = {
-        "Content-Type": "application/json",
-        "accept": "application/json",
-        "Renku-User-Id": "{0}".format(uuid.uuid4().hex),
-        "Renku-User-FullName": "Not Sam",
-        "Renku-User-Email": "not@sam.io",
-        "Authorization": f"Bearer {IT_GIT_ACCESS_TOKEN}",
-    }
-
     payload = {
-        "git_url": IT_REMOTE_REPO_URL,
+        "git_url": it_remote_repo_url,
     }
 
-    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=headers,)
+    response = svc_client.post("/cache.project_clone", data=json.dumps(payload), headers=identity_headers,)
     assert response
+
     assert {"result"} == set(response.json.keys())
 
     response = svc_client.get(
@@ -384,7 +368,8 @@ def test_clone_projects_invalid_headers(svc_client):
     assert {"error"} == set(response.json.keys())
     assert INVALID_HEADERS_ERROR_CODE == response.json["error"]["code"]
 
-    response = svc_client.get("/cache.project_list", headers=headers)
+    response = svc_client.get("/cache.project_list", headers=identity_headers)
+
     assert response
     assert {"result"} == set(response.json.keys())
     assert 1 == len(response.json["result"]["projects"])
@@ -443,7 +428,7 @@ def test_upload_zip_archive(datapack_zip, svc_client_with_repo):
 @pytest.mark.service
 @pytest.mark.integration
 def test_upload_tar_unpack_archive(datapack_tar, svc_client_with_repo):
-    """Upload zip archive with unpack."""
+    """Upload tar archive with unpack."""
     svc_client, headers, project_id, _ = svc_client_with_repo
     headers.pop("Content-Type")
 
@@ -484,7 +469,7 @@ def test_upload_tar_unpack_archive(datapack_tar, svc_client_with_repo):
 @pytest.mark.service
 @pytest.mark.integration
 def test_upload_tar_archive(datapack_tar, svc_client_with_repo):
-    """Upload zip archive."""
+    """Upload tar archive."""
     svc_client, headers, project_id, _ = svc_client_with_repo
     headers.pop("Content-Type")
 
@@ -506,6 +491,72 @@ def test_upload_tar_archive(datapack_tar, svc_client_with_repo):
         assert not file_["unpack_archive"]
 
 
+@pytest.mark.service
+@pytest.mark.integration
+def test_upload_gz_unpack_archive(datapack_gz, svc_client_with_repo):
+    """Upload gz archive with unpack."""
+    svc_client, headers, project_id, _ = svc_client_with_repo
+    headers.pop("Content-Type")
+
+    response = svc_client.post(
+        "/cache.files_upload",
+        data=dict(file=(io.BytesIO(datapack_gz.read_bytes()), datapack_gz.name, "application/x-gzip"),),
+        query_string={"unpack_archive": True, "override_existing": True,},
+        headers=headers,
+    )
+
+    assert response
+
+    assert 200 == response.status_code
+    assert {"result"} == set(response.json.keys())
+    assert response.json["result"]["files"]
+
+    for file_ in response.json["result"]["files"]:
+        assert not file_["is_archive"]
+        assert not file_["unpack_archive"]
+
+    response = svc_client.get("/cache.files_list", headers=headers,)
+
+    assert response
+    assert 200 == response.status_code
+    assert {"result"} == set(response.json.keys())
+    assert response.json["result"]["files"]
+
+    dirs = filter(lambda x: x["is_dir"], response.json["result"]["files"])
+    assert list(dirs)
+
+    files = filter(lambda x: not x["is_dir"], response.json["result"]["files"])
+    assert list(files)
+
+    paths = [_file["relative_path"] for _file in files]
+    assert sorted(paths) == paths
+
+
+@pytest.mark.service
+@pytest.mark.integration
+def test_upload_gz_archive(datapack_gz, svc_client_with_repo):
+    """Upload gz archive."""
+    svc_client, headers, project_id, _ = svc_client_with_repo
+    headers.pop("Content-Type")
+
+    response = svc_client.post(
+        "/cache.files_upload",
+        data=dict(file=(io.BytesIO(datapack_gz.read_bytes()), datapack_gz.name, "application/x-gzip"),),
+        query_string={"unpack_archive": False, "override_existing": True,},
+        headers=headers,
+    )
+
+    assert response
+
+    assert 200 == response.status_code
+    assert {"result"} == set(response.json.keys())
+    assert 1 == len(response.json["result"]["files"])
+
+    for file_ in response.json["result"]["files"]:
+        assert file_["is_archive"]
+        assert not file_["unpack_archive"]
+
+
 @pytest.mark.service
 @pytest.mark.integration
 def test_field_upload_resp_fields(datapack_tar, svc_client_with_repo):
@@ -550,9 +601,11 @@ def test_field_upload_resp_fields(datapack_tar, svc_client_with_repo):
 @pytest.mark.integration
 def test_execute_migrations(svc_client_setup):
     """Check execution of all migrations."""
-    svc_client, headers, project_id, _ = svc_client_setup
+    svc_client, headers, project_id, _, _ = svc_client_setup
 
-    response = svc_client.post("/cache.migrate", data=json.dumps(dict(project_id=project_id)), headers=headers)
+    response = svc_client.post(
+        "/cache.migrate", data=json.dumps(dict(project_id=project_id, skip_docker_update=True)), headers=headers
+    )
 
     assert 200 == response.status_code
     assert response.json["result"]["was_migrated"]
@@ -565,7 +618,7 @@ def test_execute_migrations(svc_client_setup):
 @pytest.mark.integration
 def test_execute_migrations_job(svc_client_setup):
     """Check execution of all migrations."""
-    svc_client, headers, project_id, _ = svc_client_setup
+    svc_client, headers, project_id, _, _ = svc_client_setup
 
     response = svc_client.post(
         "/cache.migrate", data=json.dumps(dict(project_id=project_id, is_delayed=True)), headers=headers
@@ -580,27 +633,35 @@ def test_execute_migrations_job(svc_client_setup):
 @pytest.mark.integration
 def test_check_migrations_local(svc_client_setup):
     """Check if migrations are required for a local project."""
-    svc_client, headers, project_id, _ = svc_client_setup
+    svc_client, headers, project_id, _, _ = svc_client_setup
 
     response = svc_client.get("/cache.migrations_check", query_string=dict(project_id=project_id), headers=headers)
     assert 200 == response.status_code
 
     assert response.json["result"]["migration_required"]
+    assert not response.json["result"]["template_update_possible"]
+    assert not response.json["result"]["docker_update_possible"]
     assert response.json["result"]["project_supported"]
     assert response.json["result"]["project_version"]
     assert response.json["result"]["latest_version"]
+    assert "template_source" in response.json["result"]
+    assert "template_ref" in response.json["result"]
+    assert "template_id" in response.json["result"]
+    assert "automated_template_update" in response.json["result"]
 
 
 @pytest.mark.service
 @pytest.mark.integration
-def test_check_migrations_remote(svc_client_setup, it_remote_repo):
+def test_check_migrations_remote(svc_client_setup, it_remote_repo_url):
     """Check if migrations are required for a remote project."""
-    svc_client, headers, _, _ = svc_client_setup
+    svc_client, headers, _, _, _ = svc_client_setup
 
-    response = svc_client.get("/cache.migrations_check", query_string=dict(git_url=it_remote_repo), headers=headers)
+    response = svc_client.get("/cache.migrations_check", query_string=dict(git_url=it_remote_repo_url), headers=headers)
     assert 200 == response.status_code
 
     assert response.json["result"]["migration_required"]
+    assert not response.json["result"]["template_update_possible"]
+    assert not response.json["result"]["docker_update_possible"]
     assert response.json["result"]["project_supported"]
     assert response.json["result"]["project_version"]
     assert response.json["result"]["latest_version"]
@@ -616,4 +677,113 @@ def test_check_no_migrations(svc_client_with_repo):
 
     assert 200 == response.status_code
     assert not response.json["result"]["migration_required"]
+    assert not response.json["result"]["template_update_possible"]
+    assert not response.json["result"]["docker_update_possible"]
     assert response.json["result"]["project_supported"]
+
+
+@pytest.mark.service
+@pytest.mark.integration
+@pytest.mark.serial
+@flaky(max_runs=10, min_passes=1)
+def test_cache_is_reset_after_failing_push(svc_protected_old_repo):
+    """Check cache state is reset after pushing to a protected branch fails."""
+    svc_client, headers, project_id, cache, user = svc_protected_old_repo
+
+    project = cache.get_project(user, project_id)
+    repo = Repo(path=project.abs_path)
+    commit_sha_before = repo.head.object.hexsha
+    active_branch_before = repo.active_branch.name
+
+    response = svc_client.post(
+        "/cache.migrate", data=json.dumps(dict(project_id=project_id, skip_docker_update=True)), headers=headers
+    )
+    assert 200 == response.status_code
+    assert response.json["result"]["was_migrated"]
+
+    project = cache.get_project(user, project_id)
+    repo = Repo(path=project.abs_path)
+
+    assert commit_sha_before == repo.head.object.hexsha
+    assert active_branch_before == repo.active_branch.name
+
+
+@pytest.mark.service
+@pytest.mark.integration
+@pytest.mark.serial
+@flaky(max_runs=10, min_passes=1)
+def test_migrating_protected_branch(svc_protected_old_repo):
+    """Check migrating on a protected branch does not change cache state."""
+    svc_client, headers, project_id, _, _ = svc_protected_old_repo
+
+    response = svc_client.get("/cache.migrations_check", query_string=dict(project_id=project_id), headers=headers)
+    assert 200 == response.status_code
+    assert response.json["result"]["migration_required"]
+
+    response = svc_client.post(
+        "/cache.migrate", data=json.dumps(dict(project_id=project_id, skip_docker_update=True)), headers=headers
+    )
+
+    assert 200 == response.status_code
+    assert response.json["result"]["was_migrated"]
+    assert any(
+        m.startswith("Successfully applied") and m.endswith("migrations.") for m in response.json["result"]["messages"]
+    )
+
+    response = svc_client.get("/cache.migrations_check", query_string=dict(project_id=project_id), headers=headers)
+    assert 200 == response.status_code
+    assert response.json["result"]["migration_required"]
+
+
+@pytest.mark.service
+@pytest.mark.integration
+@pytest.mark.serial
+@flaky(max_runs=1, min_passes=1)
+def test_cache_gets_synchronized(local_remote_repository, directory_tree):
+    """Test that the cache stays synchronized with the remote repo."""
+    from renku.core.management.client import LocalClient
+    from renku.core.models.provenance.agents import Person
+
+    svc_client, identity_headers, project_id, remote_repo, remote_repo_checkout = local_remote_repository
+
+    remote_name = remote_repo_checkout.active_branch.tracking_branch().remote_name
+    remote = remote_repo_checkout.remotes[remote_name]
+
+    client = LocalClient(remote_repo_checkout.working_dir)
+
+    with client.commit(commit_message="Create dataset"):
+        with client.with_dataset("my_dataset", create=True) as dataset:
+            dataset.creators = [Person(name="me", email="me@example.com", id="me_id")]
+
+    remote.push()
+
+    params = {
+        "project_id": project_id,
+    }
+
+    response = svc_client.get("/datasets.list", query_string=params, headers=identity_headers,)
+
+    assert response
+    assert 200 == response.status_code
+
+    assert {"datasets"} == set(response.json["result"].keys())
+    assert 1 == len(response.json["result"]["datasets"])
+
+    payload = {
+        "project_id": project_id,
+        "name": uuid.uuid4().hex,
+    }
+
+    response = svc_client.post("/datasets.create", data=json.dumps(payload), headers=identity_headers,)
+
+    assert response
+    assert 200 == response.status_code
+    assert {"name", "remote_branch"} == set(response.json["result"].keys())
+
+    remote.pull()
+
+    datasets = client.datasets.values()
+    assert 2 == len(datasets)
+
+    assert any(d.name == "my_dataset" for d in datasets)
+    assert any(d.name == payload["name"] for d in datasets)
